package algds.contest;

/**
 * Dieses Interface dient als allgemeines Sammelbecken fuer
 * die dem Hamster bekannten Konstanten.
 * Es wird von Hamster implementiert und diese Werte
 * stehen damit den Abkoemmlingen von Hamster direkt zur Verfuegung.
 *
 * @author Christian Semrau
 * @see Hamster
 */
public interface Constants {

	/** Das betrachtete Feld ist leer (= 0).
	 * Wenn ein betrachtetes Feld leer ist, also keinen Mais
	 * und kein Beamer enthaelt und keine Wand im Blick ist,
	 * so wird EMPTY geliefert.
	 * Der Wert EMPTY kann nicht mit einem bitweisen Vergleich getestet
	 * werden, sondern direkt mit <code>(look()==EMPTY)</code>.
	 * @see Hamster#look()
	 * @see Hamster#lookCurrent()
	 */
	public final static int EMPTY = 0;

	/** Auf dem betrachteten Feld befindet sich Mais (= 1).
	 * Dieser Wert muss bitweise aus dem look()-Ergebnis extrahiert werden:
	 * Ein Feld enthaelt Mais, genau dann wenn der Ausdruck
	 * <code>((look() & CORN) != 0)</code> true liefert.
	 * @see Hamster#look()
	 * @see Hamster#lookCurrent()
	 */
	public final static int CORN = 1;

	/** Der Blick wird von einer Wand blockiert (= 2).
	 * Bitweiser Vergleich wie {@link #CORN}.
	 * @see Hamster#look()
	 */
	public final static int WALL = 2;

	/** Das betrachtete Feld enhaelt einen Beamer (= 4).
	 * Bitweiser Vergleich wie {@link #CORN}.
	 * @see Hamster#look()
	 * @see Hamster#lookCurrent()
	 */
	public final static int BEAMER = 4;

	/** Maximale Gesamtgroesse der gleichzeitig
	 * transportierbaren Koerner (= 20).
	 * Sozusagen das "Backenvolumen" des Hamsters.
	 * @see Hamster#take(int, int)
	 */
	public final static int CORNCARRY_MAX = 20;

	/** Maximale Groesse der Koerner im Labyrinth (= 7).
	 * Moegliche Korngroessen sind 1,2...,MAX_CORNSIZE
	 * @see Hamster#take(int, int)
	 * @see Hamster#drop(int, int)
	 * @see Hamster#eat(int, int)
	 * @see Hamster#getCarriedCorn(int)
	 * @see Hamster#getCollectedCorn(int)
	 * @see Hamster#getAvailableCorn(int)
	 */
	public final static int CORNSIZE_MAX = 7;

	/** Gesamtgroesse der maximal auf ein Feld
	 * passenden Koerner (= 500). Die einzige Ausnahme von der
	 * Platzbeschraenkung ist das Heimatfeld.
	 * @see Hamster#drop(int, int)
	 */
	public final static int CORNFIELD_MAX = 500;

	/** Anfaenglicher Energievorrat des Hamsters (= 4000). */
	public final static int ENERGY_START = 4000;

	/** Maximaler Energievorrat des Hamsters (= 20000).
	 * @see Hamster#eat(int, int)
	 */
	public final static int ENERGY_MAX = 20000;

	/** Energieaenderung pro Schritt (= -4).
	 * @see Hamster#forward()
	 */
	public final static int ENERGY_STEP = -4;

	/** Energieaenderung pro Drehung (= -1).
	 * @see Hamster#turn(int)
	 */
	public final static int ENERGY_TURN = -1;

	/** Energieaengerung pro Kollision (= -19).
	 * Die Drehung zieht einen weiteren Energiepunkt ab.
	 * @see Hamster#forward()
	 */
	public final static int ENERGY_COLLISION = -19;

	/** Energieaengerung pro gefressenem Korn der Groesse 1 (= +200).
	 * @see Hamster#eat(int, int)
	 */
	public final static int ENERGY_CORN = 200;

	/** Punkte fuer heimgebrachte Koerner der Groesse 1 (= +30).
	 * @see Hamster#getPoints()
	 */
	public final static int POINTS_CORN = 30;

	/** Punkte fuer einen Schritt des Hamsters (= -1).
	 * @see Hamster#forward()
	 * @see Hamster#getPoints()
	 */
	public final static int POINTS_STEP = -1;

	/** Punkte fuer Kollision (= -20).
	 * @see Hamster#forward()
	 * @see Hamster#getPoints()
	 */
	public final static int POINTS_COLLISION = -20;

	/** Punkte fuer Verhungern (= -200).
	 * @see Hamster#getPoints()
	 */
	public final static int POINTS_DIE = -200;

	/** Punkte fuer Beenden ausserhalb des Heimatfeldes (= -100).
	 * @see Hamster#getPoints()
	 */
	public final static int POINTS_NOTHOME = -100;

	/** Punkte fuer Absturz oder Timeout (= -5000).
	 * @see Hamster#getPoints()
	 */
	public final static int POINTS_CRASH = -5000;

	/** Wartezeit in Millisekunden fuer Entscheidung ob Endlosschleife
	 * (= 3 Minuten).
	 * Wenn innerhalb dieser Zeit keine der Methoden
	 * {@link Hamster#forward()}, {@link Hamster#beamMeUp()} oder
	 * {@link Hamster#turn(int)} aufgerufen wird, dann wird der Lauf
	 * beendet und der Hamster mit {@link #POINTS_CRASH} Punkten
	 * bestraft.
	 */
	public final static long INFINITE_LOOP = 3*60*1000;

	/** Zeit in Millisekunden fuer den gesamten Durchlauf durch
	 * ein Labyrinth (= 120 Minuten).
	 * Wenn innerhalb dieser Zeit der Lauf nicht beendet ist,
	 * dann wird der Lauf beendet und der Hamster mit
	 * {@link #POINTS_CRASH} Punkten bestraft.
	 */
	public final static long TOTAL_RUNTIME = 120*60*1000;

	/** Nach Links drehen.
	 * @see Hamster#turn(int)
	 */
	public final static int TURN_LEFT = 0;

	/** Nach Rechts drehen.
	 * @see Hamster#turn(int)
	 */
	public final static int TURN_RIGHT = 1;

	/** Blickrichtung ist nach Westen.
	 * @see Hamster#getDirection()
	 */
	public final static int DIR_WEST = 30;

	/** Blickrichtung ist nach Norden.
	 * @see Hamster#getDirection()
	 */
	public final static int DIR_NORTH = DIR_WEST+1;

	/** Blickrichtung ist nach Osten.
	 * @see Hamster#getDirection()
	 */
	public final static int DIR_EAST = DIR_WEST+2;

	/** Blickrichtung ist nach Sueden.
	 * @see Hamster#getDirection()
	 */
	public final static int DIR_SOUTH = DIR_WEST+3;
}