package algds.contest;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.Date;

/**
 * Hauptfenster der World, die Editor und Runner in einem ist.
 * <br>Creation date: (03.02.01 17:00:00)
 * @author Christian Semrau
 * @see World
 */
public class ControlFrame extends JFrame implements ActionListener, Logable {

	/** Datum der letzten Aenderung. */
	public static final String programDate = "12.03.2001";

	/** Programmversion. */
	public static final String programVersion = "1.1";

	/** Der Menubar. */
	JMenuBar menuBar = new JMenuBar();
	/** Die Menus. */
	JMenu
		menuMaze = new JMenu("Maze"),
		menuHamster = new JMenu("Hamster"),
		menuWorld = new JMenu("World");
	/** Die Menu-Eintraege. */
	JMenuItem
		mazeNew = new JMenuItem("New..."),
		mazeLoad = new JMenuItem("Load..."),
		mazeReload = new JMenuItem("Reload"),
		mazeSave = new JMenuItem("Save"),
		mazeSaveAs = new JMenuItem("Save as..."),
		mazeRedraw = new JMenuItem("Make Visited"),
		mazeProperties = new JMenuItem("Properties..."),
		hamsterLoad = new JMenuItem("Load..."),
		hamsterReload = new JMenuItem("Reload"),
		worldAbout = new JMenuItem("About..."),
		worldClearlog = new JMenuItem("Clear log"),
		worldPack = new JMenuItem("Pack"),
		worldExit = new JMenuItem("Exit");
	JCheckBoxMenuItem
		hamsterVerbose = new JCheckBoxMenuItem("Verbose", isHamsterVerbose),
		hamsterMute = new JCheckBoxMenuItem("Mute", isHamsterMute),
		hamsterFollow = new JCheckBoxMenuItem("Follow", true);

	/** Startbutton */
	JButton buttonStart = new JButton(new ImageIcon (getClass().getResource("start.gif")));
	/** Stopbutton */
	JButton buttonStop = new JButton(new ImageIcon (getClass().getResource("stop.gif")));

	/** Das Editorpanel. */
	ControlPanelEditor editor = new ControlPanelEditor(this);
	/** Die Log-Area. */
	JTextArea logText = new JTextArea(6, 60);

	/** Verzeichnis, in dem die Labyrinthe abgelegt werden sollten. */
	String mazeDir = System.getProperty("user.dir") + File.separator + "mazes" + File.separator;

	/** Namen des Labyrinths, des Hamsters. */
	String mazeName = "", hamsterName = "";
	/** Name des Autors fuer neue Labyrinthe. */
	static String creatorName = "";
	/** Das Labyrinth. */
	Maze maze = null;
	/** Die MazeEngine. Wird mit jedem geladenen Labyrinth neu konstruiert. */
	EngineX engineX = null;
	/** Der Hamster. */
	Hamster hamster = null;
	
	long startZeit, endZeit;

	/** Dialog zum Laden und Speichern der Labyrinthe. */
	FileDialog fileDialog = null;
	/** Kann der Hamster gestartet werden. */
	boolean hamsterStartable = false;
	/** Properties-Dialog */
	PropertiesDialog properties = null;

	/** Voreingestellte Wartezeit.
	 */
 	static int defaultSleep = 50;
	/** Voreingestellte Zellengroesse. Aenderungen werden erst beim
	 * Neuladen wirksam.
	 */
	static int defaultCellSize = 20;
	/** Gibt an, ob das Labyrinth-Fenster seine Groesse
	 * automatisch an die Gegebenheiten anpassen soll.
	 * @see EngineX#checkSize()
	 */
	static boolean engineAutoResize = false;
	/**
	 * Gibt an, ob das Labyrinth-Fenster dem ControlFrame folgen soll.
	 */
	static boolean engineDocking = false;
	/**
	 * Gibt an, ob die Systemspezifischen Dialoge zum
	 * Laden und Speichern verwendet werden sollen, oder
	 * ob ein JFileChooser benutzt werden soll.
	 */
	static boolean systemDialogs = true;

	/** MouseListener fuer den Editor.
	 * Reagiert auf Ereignisse im panelMaze der EngineX.
	 */
	EngineXMouse mouselistener = null;

	/** Der Hamster soll ausfuehrliche Meldungen geben.
	 * Der Wert dieser Variablen wird an engineX.verbose uebergeben, der 
	 * von {@link Hamster#verbose()} dem Hamster uebermittelt wird.
	 */
	static boolean isHamsterVerbose = false;

	/** Der Hamster soll ueberhaupt Meldungen geben.
	 * Der Wert dieser Variablen wird an engineX.mute uebergeben, der 
	 * direkt von {@link Hamster#log(java.lang.String)} verwendet wird.
	 */
	static boolean isHamsterMute = false;

	/** Beim naechsten Klick auf Stop wird der Hamster gekillt. */
	boolean nextStopKills = false;

	/**
	 * Zum Hamsterladen den Standard-Loader verwenden.
	 * true: Hamster mit dem Standardloader laden. Funktioniert in allen
	 * IDEs, bemerkt aber Neukompilierungen des Hamsters nicht.
	 * false: Hamster mit dem eigenen Loader direkt von der Platte laden.
	 * Bemerkt Neukompilierungen des Hamsters auch nicht :-(
	 * Funktioniert nicht in allen IDEs (da in einigen die Klassen nicht
	 * sofort auf Platte gespeichert werden).
	 */
	static boolean standardLoader = true;

/** Verteilt die Events auf die entsprechenden Methoden. */
public void actionPerformed(ActionEvent ev) {
	try{

	if (ev.getSource()==mazeNew)
		mazeNew_actionPerformed();
	if (ev.getSource()==mazeLoad)
		mazeLoad_actionPerformed();
	if (ev.getSource()==mazeReload)
		mazeReload_actionPerformed();
	if (ev.getSource()==mazeSave)
		mazeSave_actionPerformed();
	if (ev.getSource()==mazeSaveAs)
		mazeSaveAs_actionPerformed();
	if (ev.getSource()==mazeRedraw)
		mazeRedraw_actionPerformed();
	if (ev.getSource()==mazeProperties)
		mazeProperties_actionPerformed();

	if (ev.getSource()==hamsterLoad)
		hamsterLoad_actionPerformed();
	if (ev.getSource()==hamsterReload)
		hamsterReload_actionPerformed();
	if (ev.getSource()==hamsterVerbose)
		hamsterVerbose_actionPerformed();
	if (ev.getSource()==hamsterMute)
		hamsterMute_actionPerformed();
	if (ev.getSource()==hamsterFollow)
		hamsterFollow_actionPerformed();

	if (ev.getSource()==worldAbout)
		worldAbout_actionPerformed();
	if (ev.getSource()==worldClearlog)
		worldClearlog_actionPerformed();
	if (ev.getSource()==worldPack){
		pack();
		if (engineX!=null){
			engineX.checkSize();
			engineX.drawMaze();
		}
	}
	if (ev.getSource()==worldExit)
		worldExit_actionPerformed();

	if (ev.getSource()==buttonStart)
		buttonStart_actionPerformed();
	if (ev.getSource()==buttonStop)
		buttonStop_actionPerformed();

	}catch(Exception exc){
		exc.printStackTrace(System.out);
	}
}

/**
 * Hamstername wurde ausgewaehlt, nun laden wir ihn.
 * @param name java.lang.String
 */
public void choseHamster(String name) {
	//logln("Control: Load hamster: "+name);

	Hamster h = Utils.loadHamster(name,this);
	if(h==null){
		logln("Control: Cannot load hamster.");
		return;
	}

	hamster = h;
	hamsterName = name;
	logln("Control: Hamster "+hamster.getName()+" loaded.");

	if (engineX!=null){
		// wenn schon ein Labyrinth geladen ist, Hamster aussetzen
		engineX.setHamster(hamster);
		engineX.setTitle(hamster.getName()+" in "+mazeName);
	}
	// Hamster neu geladen, kann also gestartet werden,
	// wenn ein Labyrinth auch geladen ist
	hamsterStartable = true;

	updateState();
}

/**
 * Erzeugt eine neue EngineX.
 */
public void createEngine() {
	//logln("Control: Create Engine...");
	Point oldLoc = null;
	if (engineX!=null){
		// alten MouseListener entfernen
		if (mouselistener!=null){
			engineX.panelMaze.removeMouseListener(mouselistener);
			engineX.panelMaze.removeMouseMotionListener(mouselistener);
		}

		// darf nicht getLocationOnScreen sein,
		// weil die Position mit setLocation wieder gesetzt wird
		oldLoc = engineX.frameMaze.getLocation();

		// altes Fenster schliessen
		engineX.disposeFrame();
	}

	// neue Engine konstruieren
	engineX = new EngineX(maze, this){
		public void hamsterStopped(){
			super.hamsterStopped();
			hamsterHasStopped();
		}
		public void log(String s){
			super.log(s);
			// DEBUG: das kann irgendwann raus
			System.out.print(s);
		}
	};

	engineX.setLog(logText);
	engineX.verbose = isHamsterVerbose;
	engineX.mute = isHamsterMute;

	// alles im Labyrinth als bekannt markieren
	maze.setVisClear();

	String title = mazeName;
	if (hamster!=null){
		title = hamster.getName()+" in "+getMazeName();
	}
	// neues Fenster oeffnen
	// false: Fenster kann nicht geschlossen werden
	// oldLoc: alte Position einnehmen
	engineX.openWindow(0,title,false,oldLoc,defaultCellSize);

	// wenn schon ein Hamster da ist, den ins Labyrinth setzen
	if (hamster!=null){
		engineX.setHamster(hamster);
		// hier darf der Hamster nicht als startable markiert werden,
		// denn es ist vielleicht der bereits gelaufene Hamster
	}

	// neuen MouseListener und MouseMotionListener registrieren
	mouselistener = new EngineXMouse(this, engineX);
	engineX.panelMaze.addMouseListener(mouselistener);
	engineX.panelMaze.addMouseMotionListener(mouselistener);
}

/**
 * Liefert den Namen der Hamsterklasse.
 * @return java.lang.String
 */
public String getHamsterName() {
	if (hamsterName==null) hamsterName="";
	return hamsterName;
}

/**
 * Hamster hat gestoppt.
 * Hamster wird NICHT neu geladen.
 */
public void hamsterHasStopped() {
	//logln("Control: Hamster has stopped.");
	int points = engineX.getPoints();
	int steps = engineX.getSteps();
	int collisions = engineX.getCollisions();
	int energy = engineX.getEnergy();
	int turns = engineX.getTurns();
	int corn = engineX.getCollectedCorn(0);
	int ate = engineX.getAte();

	endZeit = (new Date()).getTime();

	logln(
		"Hamster: "+hamster.getName()+
		", Maze: "+getMazeName() +
		" / Points: " + points +
		", Steps: " + steps +
		", Turns: " + turns +
		", Corn: " + corn +
		", Collisions: " + collisions +
		", Energy: " + energy +
		", Ate: " + ate +
		", Time: " + (endZeit-startZeit)/1000);

	engineX.setTitle(hamster.getName()+" in "+getMazeName());
	// Hamster kann nicht nochmal gestartet werden, sondern muss erst
	// neu geladen werden
	hamsterStartable = false;
	nextStopKills = false;
	updateState();
}

/**
 * Initialisiert den Inhalt des Frames.
 */
protected void initialize() {
	// den logText in ein ScrollPanel packen
	JScrollPane scroll = new JScrollPane(logText,
		JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
		JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

	// Unterpanel fuer Start und Stop
	JPanel p = new JPanel();
	p.add(buttonStart);
	p.add(buttonStop);

	getContentPane().add(p, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
		,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
	getContentPane().add(editor, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
		,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
	getContentPane().add(scroll, new GridBagConstraints(0, 1, 2, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(1, 1, 1, 1), 0, 0));

	buttonStart.setToolTipText("starts the Hamster");
	buttonStop.setToolTipText("stops the Hamster");
	buttonStart.addActionListener(this);
	buttonStop.addActionListener(this);
}

/**
 * Initialisiert das Menu und registriert die ActionListener des Menus.
 */
protected void initializeMenu() {
	menuMaze.add(mazeNew);
	menuMaze.addSeparator();
	menuMaze.add(mazeLoad);
	menuMaze.add(mazeReload);
	menuMaze.addSeparator();
	menuMaze.add(mazeSave);
	menuMaze.add(mazeSaveAs);
	menuMaze.addSeparator();
	menuMaze.add(mazeRedraw);
	menuMaze.add(mazeProperties);

	menuHamster.add(hamsterLoad);
	menuHamster.add(hamsterReload);
	menuHamster.addSeparator();
	menuHamster.add(hamsterMute);
	menuHamster.add(hamsterVerbose);
	menuHamster.add(hamsterFollow);

	menuWorld.add(worldAbout);
	menuWorld.addSeparator();
	menuWorld.add(worldClearlog);
	menuWorld.add(worldPack);
	menuWorld.addSeparator();
	menuWorld.add(worldExit);

	menuBar.add(menuWorld);
	menuBar.add(menuMaze);
	menuBar.add(menuHamster);
	this.setJMenuBar(menuBar);

	hamsterVerbose.setState(isHamsterVerbose);
	hamsterMute.setState(isHamsterMute);

	mazeNew.addActionListener(this);
	mazeLoad.addActionListener(this);
	mazeReload.addActionListener(this);
	mazeSave.addActionListener(this);
	mazeSaveAs.addActionListener(this);
	mazeRedraw.addActionListener(this);
	mazeProperties.addActionListener(this);

	hamsterLoad.addActionListener(this);
	hamsterReload.addActionListener(this);
	hamsterVerbose.addActionListener(this);
	hamsterMute.addActionListener(this);
	hamsterFollow.addActionListener(this);

	worldAbout.addActionListener(this);
	worldClearlog.addActionListener(this);
	worldPack.addActionListener(this);
	worldExit.addActionListener(this);
}

/**
 * 
 * @param s java.lang.String
 */
public void log(String s) {
	// ist der Cursor am Ende
	boolean atEnd = logText.getCaretPosition()==logText.getText().length();
	logText.append(s);
	if (atEnd)
		// Cursor wieder ans Ende setzen
		logText.setCaretPosition(logText.getText().length());

	System.out.print(s);
}

/**
 * Haengt an die auszugebende Nachricht einen Zeilenwechsel an.
 * @param s java.lang.String
 * @see #log(java.lang.String)
 */
public void logln(String s) {
	log (s+"\n");
}

/**
 * Laedt den Hamster neu.
 */
public void reloadHamster() {
	if (getHamsterName().length()==0)
		return;

	Hamster h = Utils.loadHamster(hamsterName,this);
	if(h==null){
		logln("Control: Cannot load hamster.");
		return;
	}

	hamster = h;

	logln("Control: Hamster "+hamster.getName()+" reloaded.");

	if (engineX!=null){
	// wenn schon ein Labyrinth geladen ist, Hamster aussetzen
		engineX.setHamster(hamster);
		engineX.setTitle(hamster.getName()+" in "+mazeName);
		// Hamster neu geladen, Labyrinth ist auch da,
		// also kann der Hamster gestartet werden
		hamsterStartable = true;
	}

	updateState();
}

/**
 * Checkt, ob die Eintraege des Menus auswaehlbar sein sollen.
 */
public void updateMenu() {

	if (engineX!=null && engineX.running){
		mazeNew.setEnabled(false);
		mazeLoad.setEnabled(false);
		mazeReload.setEnabled(false);
		mazeSave.setEnabled(false);
		mazeSaveAs.setEnabled(false);
		hamsterLoad.setEnabled(false);
		hamsterReload.setEnabled(false);
	}else{
		mazeNew.setEnabled(true);
		mazeLoad.setEnabled(true);
		if (mazeName!=null && mazeName.length()>0)
			mazeReload.setEnabled(true);
		if (maze!=null){
			mazeSave.setEnabled(true);
			mazeSaveAs.setEnabled(true);
		}else{
			mazeSave.setEnabled(false);
			mazeSaveAs.setEnabled(false);
		}
		hamsterLoad.setEnabled(true);
		if (hamsterName!=null && hamsterName.length()>0)
			hamsterReload.setEnabled(true);
	}

	if (engineX==null){
		mazeRedraw.setEnabled(false);
		hamsterFollow.setEnabled(false);
		hamsterFollow.setState(true);
	}else{
		mazeRedraw.setEnabled(true);
		hamsterFollow.setEnabled(true);
		hamsterFollow.setState(engineX.follow);
	}
	hamsterVerbose.setState(isHamsterVerbose);
	hamsterMute.setState(isHamsterMute);

	if (maze!=null && hamster!=null && hamsterStartable){
		if (!engineX.running){
			buttonStart.setEnabled(true);
			buttonStop.setEnabled(false);
		}else{
			buttonStop.setEnabled(true);
			buttonStart.setEnabled(false);
		}
	}else{
		buttonStart.setEnabled(false);
		buttonStop.setEnabled(false);
	}
}

/**
 * Checkt die Auswaelbarkeit der Menu-Eintraege und der
 * Komponenten des EditorPanels.
 * @see #updateMenu()
 * @see ControlPanelEditor#updateState()
 */
public void updateState() {
	updateMenu();
	editor.updateState();
}

/**
 * World constructor.
 * @param title Fenstertitel des ControlFrame
 * @param args Kommandozeilenparameter.
 * [0]=Maze
 *
 */
public ControlFrame(String title, String[] args) {
	super(title);
	addWindowListener(new WindowAdapter(){
		public void windowClosing(WindowEvent ev){
			worldExit_actionPerformed();
		}
	});

//	getWorldProperties(true);

	// Beim Programmstart die Properties laden
	PropIO.load(this);

	getContentPane().setLayout(new GridBagLayout());
	initializeMenu();
	initialize();
	updateState();
	pack();
	show();

	if (args.length>0)
	if (loadMaze(mazeDir, args[0])){
		createEngine();
		editor.mazeLoaded();
		updateState();
	}
}

/**
 * Liefert den ausgewaehlten Index der detailList des EditorPanel.
 * @return int
 * @see ControlPanelEditor#detailList
 * @see ControlPanelEditor#getDetailMode()
 */
public int getDetailMode() {
	return editor.getDetailMode();
}

/**
 * Liefert den ausgewaehlten Index der featureList des EditorPanel.
 * @return int
 * @see ControlPanelEditor#featureList
 * @see ControlPanelEditor#editMode
 */
public int getEditMode() {
	return editor.editMode;
}

/**
 * Laedt das angegebene Labyrinth.
 * Setzt bei Erfolg die Labyrinth-Variablen.
 * @return boolean true, wenn es geladen werden konnte
 * @param dir Pfad
 * @param name Dateiname
 */
public boolean loadMaze(String dir, String name) {
	name = MazeIO.addExtension(name);
	Maze m = reloadMaze(dir,name);
	if (m==null) return false;

	mazeDir = dir;
	mazeName = name;
	maze = m;
	//logln("Control: Maze loaded.");
	if (maze.author.length()>0)
		logln("Control: Maze "+name+" created by "+maze.author);

	return true;
}

/**
 * 
 * @return boolean
 * @param s java.lang.String
 */
public Maze reloadMaze(String dir, String name) {
	Maze m;
	try {
		m = new MazeIO().load(dir+name);
	} catch (Exception e) {
		logln("Control: Cannot load maze! " + e.toString());
		return null;
	}
	// Das Zuruecksetzen von Beamer.counter geschieht in MazeIO.load

	return m;
}

/**
 * 
 * @param b algds.contest.Beamer
 * @see ControlPanelEditor#selectHalfBeamer(algds.contest.Beamer)
 */
public void selectHalfBeamer(Beamer b) {
	editor.selectHalfBeamer(b);
}


/** buttonStart action performed */
public void buttonStart_actionPerformed() {
	if (hamster==null && hamsterName!=null && hamsterName.length()>0){
		reloadHamster();
	}

	if (engineX!=null && hamsterStartable && !engineX.running){
		//logln("Control: Start hamster.");

		// setzt alles auf "ungesehen"
		maze.setVisHidden();
		// setzt die Wartezeit des Hamster-Threads
		engineX.setSpeed(defaultSleep);
		// startet den Hamster, ohne Waiter
		engineX.drawMaze();
		engineX.setTitle(hamster.getName()+" in "+getMazeName()+" running");

		startZeit = (new Date()).getTime();

		engineX.runHamster(null);
	}else{
		logln("Control: Cannot start hamster.");
	}
	updateState();
}

/** buttonStop action performed.
 * Beim ersten Aufruf wird {@link EngineX#stopHamster(boolean, java.lang.String)} mit
 * <code>false</code> aufgerufen, beim naechsten Mal mit <code>true</code>.
 * Dadurch wird der Hamster beim zweiten Klick auf den Stop-Button gekillt.
 */
public void buttonStop_actionPerformed() {
	if (engineX.running){
		if (!nextStopKills){
			//logln("Control: Stop hamster.");
			engineX.stopHamster(false, "");
			nextStopKills = true;
		}else{
			logln("Control: Kill hamster.");
			engineX.stopHamster(true, "user");
		}
	}else{
		logln("Control: Cannot stop Hamster.");
	}
	updateState();
}

/** Hamster | Follow action performed */
public void hamsterFollow_actionPerformed() {
	if (engineX!=null){
		engineX.follow = !engineX.follow;
		//logln("Control: Follow Hamster = "+engineX.follow);
		hamsterFollow.setState(engineX.follow);
	}
}

/** Hamster | Load action performed */
public void hamsterLoad_actionPerformed() {

	//logln("Control: Input hamsterName...");
	String res = (String) JOptionPane.showInputDialog(this,
		"Hamster name:", "Load Hamster",
		JOptionPane.QUESTION_MESSAGE, null, null,
		hamsterName);
	if (res==null || res.length()<=0)
		return;
	choseHamster(res);
	updateState();

}

/** Hamster | Reload action performed */
public void hamsterReload_actionPerformed() {
	reloadHamster();
}

/** Maze | Load action performed */
public void mazeLoad_actionPerformed() {
	//logln("Control: Load maze...");

	String mN, mD;
	File file;

	if (systemDialogs){

	if (fileDialog == null) {
		fileDialog = new FileDialog(this);
		// Die Java API Dokumentation sagt zum
		// FileDialog.setFilenameFilter(java.io.FilenameFilter):
		//
		// "Filename filters do not function in Sun's reference
		// implementation for Windows 95, 98, or NT 4.0."
		// ... schade!
		// Dann lassen wir ihn eben komplett weg.

		//	fileDialog.setFilenameFilter(new FilenameFilter(){
		//		public boolean accept(File dir, String name){
		//			return name.toLoweCase().endsWith(".maze");
		//		}
		//	});
	}
	fileDialog.setMode(FileDialog.LOAD);
	fileDialog.setDirectory(mazeDir);
	if (mazeName != null && mazeName.length() > 0)
		fileDialog.setFile(mazeName);

	/* Bei mir (ChS) unter Windows 98 kann dieser Systemdialog nicht
	 * auf diese Weise positioniert werden. Beim anderen Dialog,
	 * dem JFileChooser, funktioniert das ebenfalls nicht.
	 * PropertiesDialog und MazeAboutBox lassen sich so platzieren.
	 */
	Dimension dlgSize = fileDialog.getPreferredSize();
	Dimension frmSize = getSize();
	Point loc = getLocation();
	fileDialog.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
	Math.max( (frmSize.height - dlgSize.height) / 2 + loc.y, loc.y));

	// Zeigt den Dialog an und wartet, bis er fertig ist
	fileDialog.show();

	mN = MazeIO.addExtension(fileDialog.getFile());
	if (mN == null) {
		//logln("Control: No Maze selected.");
		return;
	}
	mD = fileDialog.getDirectory();
	file = new File(mD, mN);

	}else{ // if (systemDialogs

	JFileChooser chooser = new JFileChooser(mazeDir);

	// Wenns den Filter im Systemdialog nicht gibt, dann gibts den
	// in dem anderen auch nicht!
	/*
	ExampleFileFilter filter = new ExampleFileFilter("maze", "Labyrinthe");
	chooser.setFileFilter(filter);
	*/
	chooser.setDialogTitle("Labyrinth oeffnen");
	chooser.setApproveButtonText("Oeffnen");
	if (mazeName != null && mazeName.length() > 0) {
		// wenn wir schon ein Labyrinth gewaehlt hatten,
		// dieses auswaehlen
		File mazeFile = new File(mazeDir + mazeName);
		chooser.setSelectedFile(mazeFile);
	}

	Dimension dlgSize = chooser.getPreferredSize();
	Dimension frmSize = getSize();
	Point loc = getLocation();
	chooser.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
	Math.max( (frmSize.height - dlgSize.height) / 2 + loc.y, loc.y));

	// Dialog oeffnen
	int returnVal = chooser.showOpenDialog(this);

	if (returnVal != JFileChooser.APPROVE_OPTION) {
		// Dialogergebnis war nicht "Oeffnen"
		//logln("Control: No Maze selected.");
		return;
	}
	file = chooser.getSelectedFile();
	// getParent() liefert den Pfad ohne Dateinamen
	// getName() liefert nur den Namen ohne Verzeichnis
	mD = file.getParent() + File.separator;
	mN = file.getName();

	}

	if (!file.exists()) {
		// Labyrinth existiert nicht

		// Bei Aenderungen hier bitte auch schauen, ob
		// mazeNew_actionPerformed
		// ebenfalls geaendert werden muss

		Maze m = newMaze();
		if (m == null) {
			//logln("Control: No Maze created.");
			return;
		}

		// die nun erzeugten Beamer werden bei 0 beginnend nummeriert
		Beamer.counter = 0;
		mazeDir = mD;
		mazeName = mN;
		maze = m;
		logln("Control: New Maze (" + m.getWidth() + "," + m.getHeight() + ") created.");

		createEngine();
		editor.mazeLoaded();
		updateState();
		return;
	} // if (!file.exists

	if (loadMaze(mD, mN)) {
		createEngine();
		editor.mazeLoaded();
	}
	updateState();
}

/** Maze | New action performed */
public void mazeNew_actionPerformed() {
	// Bei Aenderungen hier bitte auch schauen, ob
	// mazeLoad_actionPerformed
	// ebenfalls geaendert werden muss

	Maze m = newMaze();
	if (m==null){
		//logln("Control: No Maze created.");
		return;
	}

	// die nun erzeugten Beamer werden bei 0 beginnend nummeriert
	Beamer.counter = 0;
	mazeName = "";
	maze = m;
	logln("Control: Maze ("+m.getWidth()+","+m.getHeight()+") created.");

	createEngine();
	editor.mazeLoaded();
	updateState();
}

/** Maze | Properties action performed */
public void mazeProperties_actionPerformed() {
	if (properties==null){
		properties = new PropertiesDialog("Properties", this);
		Dimension dlgSize = properties.getPreferredSize();
		Dimension frmSize = getSize();
		Point loc = getLocation();
		properties.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
		Math.max( (frmSize.height - dlgSize.height) / 2 + loc.y, loc.y));
		properties.show();
	}else{
		properties.requestFocus();
	}
}

/** Maze | Redraw action performed */
public void mazeRedraw_actionPerformed() {
	if (engineX!=null){
		// setzt alles auf "gesehen"
		maze.setVisClear();
		// alles neu zeichnen
		engineX.drawMaze();
	}
}

/** Maze | Reload action performed */
public void mazeReload_actionPerformed() {
	Maze m = reloadMaze(mazeDir,mazeName);
	if (m==null) return;

	maze = m;
	//logln("Control: Maze "+mazeName+" reloaded.");

	createEngine();
	editor.mazeLoaded();
	updateState();
}

/** Maze | Save action performed */
public void mazeSave_actionPerformed() {
	if (mazeName==null || mazeName.length()==0){
		mazeSaveAs_actionPerformed();
	}else{
		try {
			new MazeIO().save(maze, mazeDir+mazeName);
		} catch (Exception e) {
			logln("Control: Cannot save maze! " + e.toString());
			return;
		}
		logln("Control: Maze "+mazeName+" saved.");

		updateState();
	}
}

/** Maze | SaveAs action performed */
public void mazeSaveAs_actionPerformed() {
	//logln("Control: Save maze as...");

	String mD, mN;

	if (systemDialogs){

	if (fileDialog == null) {
		fileDialog = new FileDialog(this);
	}
	fileDialog.setMode(FileDialog.SAVE);
	fileDialog.setDirectory(mazeDir);
	fileDialog.setFile(mazeName);
	fileDialog.show();

	mN = fileDialog.getFile();
	if (mN == null) {
		//logln("Control: No Name selected.");
		return;
	}
	mD = fileDialog.getDirectory();

	}else{

	JFileChooser chooser = new JFileChooser(mazeDir);
	chooser.setDialogTitle("Labyrinth speichern");
	chooser.setApproveButtonText("Speichern");

	if (mazeName != null & mazeName.length()>0) {
		// wenn wir schon einen Labyrinthnamen haben,
		// diesen auswaehlen
		File mazeFile = new File(mazeDir+mazeName);
		chooser.setSelectedFile(mazeFile);
	}

	// Dateiname zum Speichern auswaehlen
	int returnVal = chooser.showSaveDialog(this);

	if (returnVal != JFileChooser.APPROVE_OPTION) {
		//logln("No Maze selected.");
		return;
	}

	File file = chooser.getSelectedFile();
	// getParent() liefert den Pfad ohne Dateinamen
	// getName() liefert nur den Namen ohne Verzeichnis
	mD = file.getParent()+File.separator;
	mN = file.getName();

	}

	try {
		mN = MazeIO.addExtension(mN);
		new MazeIO().save(maze, mD+mN);
	} catch (Exception e) {
		logln("Control: Cannot save maze! " + e.toString());
		return;
	}
	mazeDir = mD;
	mazeName = mN;
	logln("Control: Maze "+mazeName+" saved.");

	engineX.setTitle(mazeName);

	updateState();
}

/**
 * 
 * @return algds.contest.Maze
 */
public Maze newMaze() {

	String res = (String) JOptionPane.showInputDialog(this,
	  "Maze size (width x height):",
	  "New Maze", JOptionPane.QUESTION_MESSAGE, null, null,
	   String.valueOf(Maze.DEFAULT_WIDTH) + "x" +
	   String.valueOf(Maze.DEFAULT_HEIGHT));

	if (res==null || res.length()<=0)
		return null;

	// kleines 'x' oder grosses 'X' ist gueltig
	int p = res.indexOf('x');
	if (p<0) p = res.indexOf('X');
	if (p<0) return null;

	String ws = res.substring(0, p).trim();
	String hs = res.substring(p + 1).trim();
	int w = 0, h = 0;
	try {
	  w = Integer.parseInt(ws);
	  h = Integer.parseInt(hs);
	} catch (NumberFormatException exc) {
	  return null;
	}

	// neues Labyrinth mit einem Level
	Maze m = new Maze (w, h);
	m.addNewLevel();

	// Autor setzen
	if (creatorName==null) creatorName = "";
	m.author = creatorName;

	return m;
}

/** World | About action performed */
public void worldAbout_actionPerformed() {
	MazeAboutBox dlg = new MazeAboutBox(this, "World");
	Dimension dlgSize = dlg.getPreferredSize();
	Dimension frmSize = getSize();
	Point loc = getLocation();
	dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
	Math.max( (frmSize.height - dlgSize.height) / 2 + loc.y, loc.y));
//	dlg.setLocation(loc);
	dlg.setModal(true);
	dlg.show();
}

/** World | Clearlog action performed */
public void worldClearlog_actionPerformed() {
	logText.setText("");
}

/** World | Exit action performed */
public void worldExit_actionPerformed() {
	// TODO:
	// Wenn das Labyrinth nicht gespeichert ist, nachfragen!!
	// Dazu muss man feststellen, ob es veraendert wurde.
	// Veraenderungen, die der Hamster machte, sollen dabei nicht zaehlen.
/*
	PropIO.save(this);
*/
	System.exit(0);
}

/**
 * 
 * @return java.lang.String
 */
public String getMazeName() {
	return (mazeName.length()>0?mazeName:"unnamed.maze");
}

/** Hamster | Verbose action performed */
public void hamsterVerbose_actionPerformed() {
	if (engineX!=null){
		isHamsterVerbose = !isHamsterVerbose;
		engineX.verbose = isHamsterVerbose;
		//logln("Control: Hamster Verbose = "+isHamsterVerbose);
		hamsterVerbose.setState(isHamsterVerbose);
	}
}

/** Hamster | Mute action performed */
public void hamsterMute_actionPerformed() {
	if (engineX!=null){
		isHamsterMute = !isHamsterMute;
		engineX.mute = isHamsterMute;
		//logln("Control: Hamster Mute = "+isHamsterMute);
		hamsterMute.setState(isHamsterMute);
	}
}

public String chooseZipEntry(String dir, String zipname){
	if (!zipname.endsWith(".zip")) return "";
	String[] names = Utils.readZipEntries(dir+zipname);
	return names[0];
}

}
