package algds.contest;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Das Panel, in dem die Komponenten des Editors liegen.
 * Es verwaltet die Komponenten, die Aenderungen werden
 * in EngineXMouse durchgefuehrt.
 * <br>Creation date: (06.02.01 00:13:29)
 * @author Christian Semrau
 * @see ControlFrame
 * @see EngineXMouse
 */
public final class ControlPanelEditor extends JPanel
implements ActionListener, ItemListener, MazeConstants, EditorConstants{
	/** Button zum Hinzufuegen von Ebenen. */
	JButton addLevel = new JButton("Add");
	/** Button zum Loeschen von Ebenen.*/
	JButton delLevel = new JButton("Del");
	/** Liste der im Labyrinth vorhandenen Ebenen. */
	JComboBox levelList = new JComboBox();
	/** Liste der Editor-Funktionen. */
	JComboBox featureList = new JComboBox();
	/** Liste der Editor-Unterfunktionen. */
	JComboBox detailList = new JComboBox();
	/** Das ControlFrame, in dem dieses Panel enthalten ist. */
	ControlFrame parent = null;

	/** Aktuell ausgewaehlte Editor-Funktion. */
	int editMode = 0;

	/** Button zum Tauschen von Ebenen.*/
	JButton swapLevel = new JButton("Swap");
	int swapLevel1, swapLevel2;
	boolean wantToSwap = false;

/**
 * ControlPanelEditor constructor comment.
 */
public ControlPanelEditor(ControlFrame parent) {
	super();
	this.parent = parent;
	initialize();
}
/**
 * 
 * @param ev java.awt.event.ActionEvent
 */
public void actionPerformed(ActionEvent ev) {
	if (ev.getSource()==swapLevel){
		if (wantToSwap){
			swapLevel2 = levelList.getSelectedIndex();
			if (swapLevel1 != swapLevel2){
				parent.maze.swapLevel(swapLevel1, swapLevel2);
				if (parent.engineX!=null)
					parent.engineX.selectLevel(swapLevel2);
			}
			wantToSwap = false;
			swapLevel.setText("Swap");
			swapLevel.setToolTipText("Swap current level with another level");
		}else{
			swapLevel1 = levelList.getSelectedIndex();
			wantToSwap = true;
			swapLevel.setText("Swap "+swapLevel1+"-");
			swapLevel.setToolTipText("Need to select another level to swap.");

		}
		return;
	}

	// wenn nicht direkt der Swap-Button gedrueckt wird,
	// dann wird nicht getauscht
	wantToSwap = false;
	swapLevel.setText("Swap");
	swapLevel.setToolTipText("Swap current level with another level");

	if (ev.getSource()==addLevel){
		if (parent.engineX!=null){
			parent.maze.addNewLevel();
			fillLevelList();
			int l = parent.maze.levels.size()-1;
			parent.maze.getLevel(l).setVisAll(SAW_ALL);
			try{
				parent.engineX.selectLevel(l);
				levelList.setSelectedIndex(l);
			}catch(IndexOutOfBoundsException exc){
			}catch(IllegalArgumentException exc){
			}
			updateState();
		}
		return;
	}
	if (ev.getSource()==delLevel){
		if (parent.engineX!=null){
			int l = levelList.getSelectedIndex();
			if (l<0 || l>=parent.maze.levels.size())
				return;
			parent.maze.removeLevel(l);
			if (l>=parent.maze.levels.size()) l--;
			fillLevelList();
			try{
				parent.engineX.selectLevel(l);
				levelList.setSelectedIndex(l);
			}catch(IndexOutOfBoundsException exc){
			}catch(IllegalArgumentException exc){
			}
			updateState();
		}
		return;
	}
}
/**
 * 
 * @param type int
 */
public void fillDetailList(int type) {
	try{
		detailList.removeAllItems();
	}catch(Exception exc){
	}

	switch(type){
	case EDIT_WALL:
		break;
	case EDIT_CORN:{
		for(int i=1; i<=Constants.CORNSIZE_MAX; i++){
			for(int s=0; s<DETAIL_CORNNUMBER.length; s++){
				detailList.addItem("Groesse "+i+" +- "+DETAIL_CORNNUMBER[s]);
			}
		}
		break;
	}
	case EDIT_BEAMER:{
		for(int i=0; i<DETAIL_HBINDEX; i++)
			detailList.addItem(S_DET_BEAMER[i]);
		if (parent.maze!=null){
			Vector hbe = parent.maze.halfBeamers;
			for (int i = 0; i < hbe.size(); i++) {
				Beamer b = (Beamer) hbe.get(i);
				detailList.addItem("Connect ("+b.X+","+b.Y+","+b.L+")");
			}
		}
		break;
	}
	case EDIT_HOME:{
		for(int i=0; i<DETAIL_HOMEINDEX; i++){
			detailList.addItem(S_DET_HOME[i]);
		}
		break;
	}
	}
}
/**
 * 
 */
public void fillFeatureList() {
	featureList.removeItemListener(this);

	try{
		featureList.removeAllItems();
	}catch(Exception exc){
	}
	for (int i=0; i<FEATURELIST_SIZE; i++)
		featureList.addItem(S_EDIT[i]);
	if (editMode>=0 && editMode<FEATURELIST_SIZE)
		featureList.setSelectedIndex(editMode);

	featureList.addItemListener(this);
}
/**
 * 
 */
public void fillLevelList() {
	levelList.removeItemListener(this);

	try{
		levelList.removeAllItems();
	}catch(Exception exc){
	}
	if (parent.maze!=null){
		int levelCount = parent.maze.getLevelCount();
		for (int i=0; i<levelCount; i++)
			levelList.addItem("Level "+i);
		try {
			levelList.setSelectedIndex(parent.engineX.viewZ);
		} catch (IndexOutOfBoundsException exc) {
		} catch (IllegalArgumentException exc) {
		}
	}
	levelList.addItemListener(this);
}
/**
 * Liefert den ausgewaehlten Eintrag der detailList.
 * @return int
 */
public int getDetailMode() {
	return detailList.getSelectedIndex();
}
/**
 * Initialisiert das Layout des Panels.
 */
protected void initialize() {

	levelList.setEditable (false);
	featureList.setEditable (false);
	detailList.setEditable (false);

	addLevel.addActionListener(this);
	delLevel.addActionListener(this);
	swapLevel.addActionListener(this);

	levelList.addItemListener(this);
	featureList.addItemListener(this);
	detailList.addItemListener(this);

	addLevel.setToolTipText("Add level on top");
	delLevel.setToolTipText("Delete current level");
	swapLevel.setToolTipText("Swap current level with another level");
	levelList.setToolTipText("Select level");
	featureList.setToolTipText("Select edit function");
	detailList.setToolTipText("Select edit subfunction");

	setLayout(new GridBagLayout());
	int sx = 0;
	add(addLevel, new GridBagConstraints(sx++, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
	add(delLevel, new GridBagConstraints(sx++, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
	add(swapLevel, new GridBagConstraints(sx++, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
	add(levelList, new GridBagConstraints(sx++, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 0, 0));
	add(featureList, new GridBagConstraints(sx++, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 0, 0));
	add(detailList, new GridBagConstraints(sx++, 0, 1, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 0, 0));

	fillLevelList();
	fillFeatureList();
	editMode = featureList.getSelectedIndex();
	fillDetailList(editMode);

	updateState();
}
/**
 * 
 * @param ev java.awt.event.ItemEvent
 */
public void itemStateChanged(ItemEvent ev) {
	if (ev.getSource()==levelList){
		int l = levelList.getSelectedIndex();
//		System.out.println("levelList selected "+l);
		if (parent.engineX!=null)
			parent.engineX.selectLevel(l);
		if (wantToSwap){
			if (swapLevel1==l){
				swapLevel.setText("Swap "+swapLevel1+"-");
				swapLevel.setToolTipText("Need to select another level to swap.");
			}else{
				swapLevel.setText("Swap "+swapLevel1+"-"+l);
				swapLevel.setToolTipText("Swap levels "+swapLevel1+" and "+l+".");
			}
		}
		return;
	}
	if (ev.getSource()==featureList){
		// neuer Modus, detailList aktualisieren
		editMode = featureList.getSelectedIndex();
		fillDetailList(editMode);
		return;
	}
	if (ev.getSource()==detailList){
		return;
	}
}
/**
 * Aktualisiert die levelList und die detailList.
 */
public void mazeLoaded() {
	fillLevelList();
	if (editMode == EDIT_BEAMER)
		fillDetailList(editMode);
}
/**
 * Wenn in featureList der Eintrag "Edit Beamer" ausgewaehlt ist,
 * wird der halbe Beamer b in der detailList ausgewaehlt.
 * @param b algds.contest.Beamer
 */
public void selectHalfBeamer(Beamer b) {
	if (editMode==EDIT_BEAMER){
		fillDetailList(editMode);
		if (b==null) return;

		Vector hbe = parent.maze.halfBeamers;
		int index = hbe.indexOf(b);
		if (index<0) return;
		index += DETAIL_HBINDEX;
		try {
			detailList.setSelectedIndex(index);
		} catch (IndexOutOfBoundsException exc) {
		} catch (IllegalArgumentException exc) {
		}
		
	}
}
/**
 * Checkt, ob die Komponenten des Editors aktivierbar sein sollen.
 */
public void updateState() {
	if (parent.maze==null || (parent.engineX!=null && parent.engineX.running)){
	// deaktivieren, wenn kein Labyrinth geladen ist,
	// oder der Hamster laeuft
		addLevel.setEnabled (false);
		delLevel.setEnabled (false);
		swapLevel.setEnabled (false);
		featureList.setEnabled (false);
		detailList.setEnabled (false);
	}else{
		addLevel.setEnabled (true);
		delLevel.setEnabled (parent.maze.getLevelCount()>1);
		swapLevel.setEnabled (parent.maze.getLevelCount()>1);
		featureList.setEnabled (true);
		detailList.setEnabled (true);
	}

	if (parent.maze==null){
	// deaktivieren, wenn kein Labyrinth geladen ist
		levelList.setEnabled (false);
	}else{
		levelList.setEnabled (true);
	}
}
}