package algds.contest;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

/**
 * Grafische Anzeige des Labyrinths.
 * <br>Stammt original aus dem
 * <br>MazeRunnerX v1.0 by Johannes Zander (io@wtal.de)
 *
 * @author Johannes Zander, Christian Semrau, Stephan Finn
 */
class EngineX extends MazeEngineImpl {

	/** Jede Bewegung einzeln anzeigen. */
	static boolean SLO_MO = false;

	static Image
	hamsterRImg = null,
	hamsterLImg = null,
	hamsterUImg = null,
	hamsterDImg = null;
	/** frameMaze wird von ControlFrame gezogen. Dabei wird nicht
	 * geprueft, ob verankert werden soll. */
	boolean ismoving = false;


	/** Fuer das Labyrinth verwendete Farben. */
	Color
	RGB_FLOOR_UNKNOWN,
	RGB_FLOOR_KNOWN,
	RGB_FLOOR_VISITED,
	RGB_FLOOR_HOME,
	RGB_FLOOR_CENTER,
	RGB_FLOOR_CORN,
	RGB_WALL_UNKNOWN,
	RGB_WALL_KNOWN,
	RGB_CORN_UNKNOWN,
	RGB_CORN_KNOWN,
	RGB_BEAMER_UNKNOWN,
	RGB_BEAMER_KNOWN,
	RGB_STATUS_BG,
	RGB_TEXT,
	RGB_HAMSTER,
	RGB_NOWALL_UNKNOWN,
	RGB_NOWALL_KNOWN;

	JFrame frameMaze;
	ImagePanel panelMaze;
	ImagePanel panelStat;
	JScrollPane scroll;
	/** Hoehe der Statuszeile */
	int stateHeight = 5*16;
	/** Koordinaten des Feldes,
	 * auf das die Maus zeigt. */
	int editX = -1, editY = -1;
	/** Groesse eines Feldes. */
	protected int fs = 20;
	/** Hoehe einer Statuszeile. */
	protected int sh = 16;

	/** Oeffnungswinkel fuer die Hamsteranzeige. */
	final static int[] alphas = {45 + 2*90, 45 + 1*90, 45 + 0*90, 45 + 3*90};
	/** Ecke des Besitzerfensters, an dem das Enginefenster haftet. */
	int anchor = GridBagConstraints.SOUTHWEST;
	/** Das Frame der Engine an den Besitzer ketten. */
	boolean anchored = false;
	/** Das ScrollPanel soll dem Hamster folgen. */
	boolean follow = true;

	/** Das ControlFrame.
	 * An dieses Fenster passt die Engine ihre Bildschirm-Position an.
	 */
	ControlFrame control = null;
	/** Wartezeit */
	int sleep = 0;
	int sleepCount = 0, sleepTimer = 0;

	/** angezeigtes Level */
	Level viewLevel;
	/** Levelnummer des angezeigten Levels */
	int viewZ;

	/** Hamster statt dem Cursor anzeigen. */
	static boolean showHamsterImg = true;

	public void drawMaze() {
		// ohne Frame braucht man auch nichts zeichnen
		if (frameMaze == null)
			return;

		updateStatus();

		Graphics mazeI = panelMaze.getImageGraphics();
		for (int y = 0; y < maze.getHeight(); y++)
			for (int x = 0; x < maze.getWidth(); x++)
				drawCell(x, y, mazeI);

		if (hasHamster() && viewZ == zpos) {
			// Wenn ein Hamster vorhanden und im angezeigten Level ist
			if (!showHamsterImg){
			mazeI.setColor(RGB_HAMSTER);
			int alpha = alphas[direction];
			int delta = 270;
			mazeI.drawArc(xpos * fs + fs/2-4, ypos * fs + fs/2-4, 8, 8, alpha, delta);
			}else{
			Image img;
			switch(direction){
			case WEST: img = hamsterLImg; break;
			case EAST: img = hamsterRImg; break;
			case NORTH: img = hamsterUImg; break;
			case SOUTH: img = hamsterDImg; break;
			default: img = hamsterUImg; break;
			}
			mazeI.drawImage(img, xpos*fs+1, ypos*fs+1, fs-2, fs-2, null);
			}
		}
		panelMaze.repaint();
	}

/**
 * Bewegt den Hamster und aktualisiert die Anzeige.
 */
public boolean forward() {
	boolean f;
	// alte Position fuer update
	int oldx = xpos, oldy = ypos;

	try {
		f = super.forward();
	} catch (HamsterDiedException exc) {
		if (viewZ == zpos)
			drawMaze();
		throw exc;
	}

	if (viewZ == zpos) {
		// wenn ein Schritt getan wurde, dann den Hamster auf dem
		// altem Feld verschwinden lassen
		if (f)
			updateCell(oldx, oldy, false);
		makeViewed();
		updateCell(xpos, ypos, true);
		updateStatus();
	}

	doSleep();

	return f;
}

	public int getSpeed() {
		return sleep/sleepTimer;
	}



	/**
	 * Laesst den Hamster schauen und aktualisiert die Anzeige.
	 * @see MazeEngineImpl#look()
	 */
	public int look() {
		int look = super.look();

		if ((look & Hamster.WALL) != 0) {
			// wenn Wand, dann aktuelles Feld erneuern,
			// denn die Wand gehoert zum aktuellen
			updateCell(xpos, ypos, true);
		} else {
			// sonst Nachbarfeld, denn das Feld wurde gesehen
			updateCell(neighX(), neighY(), false);
		}
		if (SLO_MO) doSleep(); else Thread.yield();
		return look;
	}

	/**
	 * Setzt die Geschwindigkeit.
	 * @param s Anzahl der Millisekunden
	 */
	public void setSpeed(int s) {
		if (s>0 && s<50){
			sleepTimer = 1;
			while (s*sleepTimer<50) sleepTimer++;
			sleep = s*sleepTimer;
		}else{
			sleep = s;
			sleepTimer = 1;
		}
		sleepCount = 0;
//		System.out.println("sleep:"+sleep+", sleepTimer:"+sleepTimer);
	}

	/**
	 * Dreht den Hamster und aktualisiert die Anzeige.
	 * @see MazeEngineImpl#turn(int)
	 */
	public void turn(int direction) {
		super.turn(direction);
		updateCell(xpos, ypos, true);
		updateStatus();
		if (SLO_MO) doSleep(); else Thread.yield();
	}
	/**
	 * Zeichnet die Zelle (x,y) des viewLevel und
	 * zeichnet den Hamster, wenn <code>drawHamster==true</code> und
	 * der Hamster im angezeigten Level ist.
	 */
	protected void updateCell(int x, int y, boolean drawHamster) {
		if (frameMaze == null)
			return;

		Graphics mazeI = panelMaze.getImageGraphics();
		// Feld zeichnen
		drawCell(x, y, mazeI);
		if (drawHamster && viewZ==zpos) {
			// Hamster zeichnen
			if (!showHamsterImg){
			mazeI.setColor(RGB_HAMSTER);
			int alpha = alphas[direction];
			int delta = 270;
			mazeI.drawArc(xpos * fs + fs/2-4, ypos * fs + fs/2-4, 8, 8, alpha, delta);
			}else{
			Image img;
			switch(direction){
			case WEST: img = hamsterLImg; break;
			case EAST: img = hamsterRImg; break;
			case NORTH: img = hamsterUImg; break;
			case SOUTH: img = hamsterDImg; break;
			default: img = hamsterUImg; break;
			}
			mazeI.drawImage(img, xpos*fs+1, ypos*fs+1, fs-2, fs-2, null);
			}
		}

		panelMaze.repaint(x * fs, y * fs, fs, fs);
	}
/**
 * Zeichnet die Statusanzeige neu.
 * Inclusive der Editorzeile.
 */
protected void updateStatus() {
	if (frameMaze == null)
		return;

	String stepsString = "-",
		turnsString = "-",
		pointsString = "-",
		energyString = "-",
		carryString = "-",
		collectedString = "-",
		availableString = "-",
		koordString = "(-,-,-,-)",
		collisionsString = "-";

	if (hasHamster()) {
		stepsString = "" + steps;
		turnsString = "" + turns;
		pointsString = "" + getPoints();
		energyString = "" + energy;
		carryString = "" + getCarriedCorn(0) + " (";
		for (int gr = 1; gr <= CORNSIZE_MAX; gr++) {
			carryString += getCarriedCorn(gr) + " ";
		}
		carryString += ")";

		collectedString = "" + getCollectedCorn(0) + " (";
		for (int gr = 1; gr <= CORNSIZE_MAX; gr++) {
			collectedString += getCollectedCorn(gr) + " ";
		}
		collectedString += ")";

		availableString = "" + getAvailableCorn(0) + " (";
		for (int gr = 1; gr <= CORNSIZE_MAX; gr++) {
			availableString += getAvailableCorn(gr) + " ";
		}
		availableString += ")";

		koordString = "(" + getX() + "," + getY() + "," + getLevel()
			+ "," + getDirString() + ")";
		
		collisionsString = "" + collisions;
	}

	Graphics statI = panelStat.getImageGraphics();

	statI.setColor(RGB_STATUS_BG);
	statI.fillRect(0, 0, 300, 4*sh);

	statI.setColor(RGB_TEXT);
	statI.drawString("energy: " + energyString, 4, 13);
	statI.drawString("score: " + pointsString, 4 + 200, 13);
		statI.drawString("cols: " + collisionsString, 4 + 100, 13);
	statI.drawString("carry: " + carryString, 4, 13 + sh);
	statI.drawString("steps: " + stepsString, 4 + 210, 13 + sh);
	statI.drawString("turns: " + turnsString, 4 + 210, 13 + 2*sh);
	statI.drawString("avail: " + availableString, 4, 13 + 2*sh);
	statI.drawString("coll: " + collectedString, 4, 13 + 3*sh);
	statI.drawString(koordString, 4 + 200, 13 + 3*sh);
	panelStat.repaint(0, 0, 300, 4*sh);

	updateEditLine();
}



public boolean beamMeUp() throws HamsterDiedException {
	// Alte Position fuer update merken
	int oldz = zpos, oldx = xpos, oldy = ypos;

	// gebeamt
	boolean b;

	try {
		b = super.beamMeUp();
		if (follow) {
			viewLevel = currLevel;
			viewZ = zpos;
		}
	} catch (HamsterDiedException exc) {
		if (follow) {
			viewLevel = currLevel;
			viewZ = zpos;
		}
		// Wenn verhungert, ein letztes Mal das Labyrinth zeichnen
		if (viewZ == zpos)
			drawMaze();
		throw exc;
	}

	if (viewZ==zpos) {
	// wenn der Hamster sichtbar ist

		// Wenn das Level geaendert wurde, muss alles gezeichnet werden
		boolean completeRefresh = oldz != zpos;

		if (completeRefresh){
			drawMaze();
			makeViewed();
		}else {
			// gleiches Level, also nur alte und neue Position zeichnen
			// zwischendurch dafuer sorgen, dass der Hamster sichtbar ist
			updateCell(oldx, oldy, false);
			makeViewed();
			updateCell(xpos, ypos, true);
			updateStatus();
		}
	}

	doSleep();

	return b;
}

	/**
	 * Legt Mais ab und aktualisiert die Anzeige.
	 * @see MazeEngineImpl#drop(int,int)
	 */
	public int drop(int groesse, int anzahl) {
		int d = super.drop(groesse, anzahl);
		updateStatus();
		updateCell(xpos, ypos, true);
		if (SLO_MO) doSleep(); else Thread.yield();
		return d;
	}

	/**
	 * Frisst Mais und aktualisiert die Anzeige.
	 * @see MazeEngineImpl#eat(int,int)
	 */
	public int eat(int groesse, int anzahl) {
		int e = super.eat(groesse, anzahl);
		updateStatus();
		if (SLO_MO) doSleep(); else Thread.yield();
		return e;
	}

	/**
	 * Laesst den Hamster schauen und aktualisiert die Anzeige.
	 * @see MazeEngineImpl#lookCurrent()
	 */
	public int lookCurrent() {
		int look = super.lookCurrent();
		updateCell(xpos, ypos, true);
		if (SLO_MO) doSleep(); else Thread.yield();
		return look;
	}

	/**
	 * Nimmt Mais und aktualisiert die Anzeige.
	 * @see MazeEngineImpl#take(int,int)
	 */
	public int take(int groesse, int anzahl) {
		int t = super.take(groesse, anzahl);
		updateStatus();
		updateCell(xpos, ypos, true);
		if (SLO_MO) doSleep(); else Thread.yield();
		return t;
	}

	/**
	 * Erzeugt ein Frame, in das die beiden Panels gepackt werden.
	 * panelMaze und panelStat muessen da bereits vorhanden sein.
	 * Die beiden Images koennen erst erzeugt werden, machdem
	 * die panels in ein Frame eingebunden wurden.
	 *
	 * @param title
	 * @param exitAtClose true = Beim Schliessen Programm beenden,
	 * false = User kann das Fenster nicht selbst nicht schliessen
	 */
	public void createFrame(String title, boolean exitAtClose) {
		// wenn kein Titel angegeben ist
		if (title==null || title.length()==0)
			title = "unnamed.maze";
		// Fenster erzeugen
		frameMaze = new JFrame(title);
		// den panelMaze packen wir in ein weiteres Panel,
		// damit die Mausereignisse nur dort registriert werden,
		// wo sie interessant sind - ueber dem Labyrinth
		JPanel p2 = new JPanel(new FlowLayout(FlowLayout.LEFT,0,0));
		p2.setBackground(Color.black);
		p2.add(panelMaze);
		// in den ScrollPanel packen wir dieses Zusatzpanel
		// der Einfachheit halber lassen wir die Scrollbars immer angezeigt
		scroll = new JScrollPane(
			p2,
			JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		GridBagLayout gridBagLayout = new GridBagLayout();
		Container pane = frameMaze.getContentPane();
		pane.setLayout(gridBagLayout);

		// die beiden Komponenten ins Frame einfuegen
		pane.add(panelStat, new GridBagConstraints(
			0, 0, 1, 1, 0.0, 0.0,
			GridBagConstraints.WEST,
			GridBagConstraints.NONE,
			new Insets(0, 0, 0, 0), 0, 0));
		pane.add( scroll, new GridBagConstraints(
			0, 1, 1, 1, 1.0, 1.0,
			GridBagConstraints.WEST,
			GridBagConstraints.BOTH,
			new Insets(0, 0, 0, 0), 0, 0));

		// Groesse optimieren
		frameMaze.pack();

		// das frameMaze wird mitgeschleppt, wenn es verankert ist
		if (control!=null){
			control.addComponentListener(new EngineXcCL(this));
		} // if (control

		// Der ComponentListener prueft, ob frameMaze immer noch verankert sein soll.
		// Es wird die Fenstergroesse an die Grenzen des Bildschirms angepasst.
		frameMaze.addComponentListener(new EngineXfCL(this));

		if (exitAtClose) {
			frameMaze.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			});
		}else{
			frameMaze.setDefaultCloseOperation(
				WindowConstants.DO_NOTHING_ON_CLOSE);
		}

	}

	/**
	 * Legt die beiden Images an und fuellt sie mit der Hintergrundfarbe.
	 */
	public void createImages() {
		Graphics mazeI = panelMaze.createImage();
		mazeI.setColor(RGB_FLOOR_UNKNOWN);
		mazeI.fillRect(0, 0, maze.getWidth() * fs, maze.getHeight() * fs);

		Graphics statI = panelStat.createImage();
		statI.setColor(RGB_STATUS_BG);
		statI.fillRect(0, 0, 300, stateHeight);
	}

	/**
	 * 
	 */
	public void disposeFrame() {
		frameMaze.setVisible(false);
		frameMaze.dispose();
	}

	/**
	 * Zeichnet die Zelle (x,y) des viewLevel.
	 */
	protected void drawCell(int x, int y, Graphics mazeI) {
		// Daten der Zelle sammeln
		int vis = viewLevel.getVis(x,y);
		int borders = viewLevel.getBorders(x,y);
		Beamer b = viewLevel.getBeamer(x,y);
		int c = viewLevel.getCorn(x,y,0);
		boolean isHome = (x==maze.homex && y==maze.homey && viewZ==maze.homeLevel);
		boolean isCenter = (x==viewLevel.centerX && y==viewLevel.centerY);

		int px = x*fs, py = y*fs, px1 = (x+1)*fs-1, py1=(y+1)*fs-1;
		Color col;

		// Feldhintergrund
		col = (vis & VIS_FIELD) == 0
			? ((vis & SAW_FIELD) == 0
				? RGB_FLOOR_UNKNOWN
				: RGB_FLOOR_KNOWN)
			: (c==0
				? RGB_FLOOR_VISITED
				: RGB_FLOOR_CORN);
		mazeI.setColor(col);
		mazeI.fillRect(px, py, fs, fs);

		if (isHome){
			// Rand ziehen
			mazeI.setColor(RGB_FLOOR_HOME);
			mazeI.drawRect(px+1, py+1, fs-3, fs-3);
		}
		if (isCenter){
			// Kreuz machen
			mazeI.setColor(RGB_FLOOR_CENTER);
			mazeI.drawLine(px, py, px1, py1);
			mazeI.drawLine(px1, py, px, py1);
		}

		// Waende
		col = (borders & BORDER_NORTH) != 0
			? ((vis & SAW_NORTH) == 0
				? RGB_WALL_UNKNOWN
				: RGB_WALL_KNOWN)
			: ((vis & SAW_NORTH) == 0
				? RGB_NOWALL_UNKNOWN
				: RGB_NOWALL_KNOWN);
		mazeI.setColor(col);
		mazeI.drawLine(px, py, px1, py);

		col = (borders & BORDER_EAST) != 0
			? ((vis & SAW_EAST) == 0
				? RGB_WALL_UNKNOWN
				: RGB_WALL_KNOWN)
			: ((vis & SAW_EAST) == 0
				? RGB_NOWALL_UNKNOWN
				: RGB_NOWALL_KNOWN);
		mazeI.setColor(col);
		mazeI.drawLine(px1, py, px1, py1);

		col = (borders & BORDER_SOUTH) != 0
			? ((vis & SAW_SOUTH) == 0
				? RGB_WALL_UNKNOWN
				: RGB_WALL_KNOWN)
			: ((vis & SAW_SOUTH) == 0
				? RGB_NOWALL_UNKNOWN
				: RGB_NOWALL_KNOWN);
		mazeI.setColor(col);
		mazeI.drawLine(px, py1, px1, py1);

		col = (borders & BORDER_WEST) != 0
			? ((vis & SAW_WEST) == 0
				? RGB_WALL_UNKNOWN
				: RGB_WALL_KNOWN)
			: ((vis & SAW_WEST) == 0
				? RGB_NOWALL_UNKNOWN
				: RGB_NOWALL_KNOWN);
		mazeI.setColor(col);
		mazeI.drawLine(px, py, px, py1);

		// Mais
		if (c > 0) {
			int size = Math.min(20, c)/2+2;
			mazeI.setColor((vis & SAW_FIELD) == 0 ? RGB_CORN_UNKNOWN : RGB_CORN_KNOWN);
			mazeI.fillOval(
				px + (fs - size)/2,
				py + (fs - size)/2,
				size, size);
		}

				/** Beamer:
				 *  Das gute Aussehen beschraenkt die maximale Anzahl
				 *  der Beamer bei der kleinsten Zellgraosse auf 255!
				 *  Sollte aber eigentlich doch reichen! Oder?
				 */
		if (b != null) {
			mazeI.setColor((vis & SAW_FIELD) == 0 ? RGB_BEAMER_UNKNOWN : RGB_BEAMER_KNOWN);
			// Durch die Hex-Darstellung koennen wir 256 Beamer sauber zeichnen.
						Font f = mazeI.getFont();
						mazeI.setFont(new Font(f.getFontName(),f.getStyle(),Math.max(fs/2,10)));
						// hexString ist umgezogen in Utils
						mazeI.drawString(Utils.hexString(b.ID), px + fs/8, py + fs - (fs/5));
			mazeI.setFont(f);
		}
	}





	/**
	 * Setzt dem Hamster ins Labyrinth und zeichnet ihn.
	 * @param hamster algds.contest.Hamster
	 */
	public void setHamster(Hamster hamster) {
		// Hamster aussetzen
		super.setHamster(hamster);
		if (hamster!=null){
			// ins Level des Hamsters wechseln
			viewLevel = maze.getLevel(viewZ = zpos);
		}
		// Alles zeichnen, denn der Hamster war vielleicht
		// vorher woanders
		drawMaze();
	}

	public EngineX(Maze maze, ControlFrame control) {
		super(maze);
		this.control = control;

		if (hamsterRImg==null){
			hamsterRImg = new ImageIcon(getClass().getResource("hamsterr.gif")).getImage();
			hamsterLImg = new ImageIcon(getClass().getResource("hamsterl.gif")).getImage();
			hamsterUImg = new ImageIcon(getClass().getResource("hamsteru.gif")).getImage();
			hamsterDImg = new ImageIcon(getClass().getResource("hamsterd.gif")).getImage();
		}

		RGB_FLOOR_UNKNOWN = new Color(0xB5B5B5);
		RGB_FLOOR_KNOWN = new Color(0xE0E0E0);
		RGB_FLOOR_VISITED = new Color(0xffffff);
		RGB_FLOOR_HOME = new Color(0x000000);
		RGB_FLOOR_CENTER = new Color(0x000000);
		RGB_FLOOR_CORN = new Color(0xFFf0e0);
		RGB_NOWALL_UNKNOWN = new Color(0xB5B5B5);
		RGB_NOWALL_KNOWN = new Color(0xF0F0F0);
		RGB_WALL_UNKNOWN = new Color(0xB5B5B5);
		RGB_WALL_KNOWN = new Color(0x0000ff);
		RGB_CORN_UNKNOWN = new Color(0xB5B5B5);
		RGB_CORN_KNOWN = new Color(0xFFC000);
		RGB_BEAMER_UNKNOWN = new Color(0xB5B5B5);
		RGB_BEAMER_KNOWN = new Color(0xFF0000);
		RGB_HAMSTER = new Color(0xff0000);
		RGB_STATUS_BG = new Color(0xffffff);
		RGB_TEXT = new Color(0x000000);

		/*
		RGB_FLOOR_UNKNOWN = new Color(0xC0C0C0);
		RGB_FLOOR_KNOWN = new Color(0xE0E0E0);
		RGB_FLOOR_VISITED = new Color(0xffffff);
		RGB_FLOOR_HOME = new Color(0x000000);
		RGB_FLOOR_CENTER = new Color(0x000000);
		RGB_FLOOR_CORN = new Color(0xFFf0e0);
		RGB_NOWALL_UNKNOWN = new Color(0xE0E0E0);
		RGB_NOWALL_KNOWN = new Color(0xE0E0E0);
		RGB_WALL_UNKNOWN = new Color(0x000080);
		RGB_WALL_KNOWN = new Color(0x0000ff);
		RGB_CORN_UNKNOWN = new Color(0xDE9452);
		RGB_CORN_KNOWN = new Color(0xFFC000);
		RGB_BEAMER_UNKNOWN = new Color(0x800000);
		RGB_BEAMER_KNOWN = new Color(0xFF0000);
		RGB_HAMSTER = new Color(0xff0000);
		RGB_STATUS_BG = new Color(0xffffff);
		RGB_TEXT = new Color(0x000000);
		*/
	}

/**
 * 
 */
public void doSleep() {
	if (sleep > 0){
		if (++sleepCount >= sleepTimer){
			sleepCount = 0;
			try {
				Thread.sleep(sleep);
			} catch (InterruptedException e) {
			}
		}else
			Thread.yield();
	}else
		Thread.yield();
}

/**
 * Wird von Runner aufgerufen, wenn der Hamster gestoppt hat.
 */
public void hamsterStopped() {
	super.hamsterStopped();
	drawMaze();
}

/**
 * Scrollt so, dass der Hamster sichtbar wird.
 * Tut nichts, wenn nicht gefolgt werden soll,
 * oder der Hamster auf einer anderen Ebene ist.
 * <br>Die Feldanzeige des Mauscursors wird dabei leider nicht aktualisiert.
 */
public void makeViewed() {
	if (!follow) return;
	if (viewZ!=zpos) return;

	Point vp = scroll.getViewport().getViewPosition();
	Rectangle viewrec = scroll.getViewport().getViewRect();

	// Anzeige gross genug fuer das ganze Labyrinth
	if (viewrec.x==0 && viewrec.y==0 && viewrec.width>=maze.getWidth()*fs
		&& viewrec.height>=maze.getHeight()*fs){
			return;
		}

	int dx = 0, dy = 0;
	if (direction==WEST || direction==EAST) dx = 1; else dy = 1;
	Rectangle hamsterrec = new Rectangle(
		(xpos-dx) * fs, (ypos-dy) * fs,
		fs*(2*dx+1)-1, fs*(2*dy+1)-1);
	hamsterrec = hamsterrec.intersection(new Rectangle(
		0, 0,
		maze.getWidth()*fs-1, maze.getHeight()*fs-1));
	if (!viewrec.contains(hamsterrec)){
		Rectangle rec;
		dx = 1; dy = 1;
		rec = new Rectangle(
			(xpos-dx) * fs - vp.x, (ypos-dy) * fs - vp.y,
			fs*(2*dx+1)-1, fs*(2*dy+1)-1);
		scroll.getViewport().scrollRectToVisible(rec);
		dx = 3; dy = 3;
		rec = new Rectangle(
			(xpos-dx) * fs - vp.x, (ypos-dy) * fs - vp.y,
			fs*(2*dx+1)-1, fs*(2*dy+1)-1);
		scroll.getViewport().scrollRectToVisible(rec);
	}
}

/**
 * Setzt das Engine-Fenster (frameMaze) an die linke untere Ecke
 * des ControlFrame (wenn eins da ist).
 */
public void positionFrame() {
	if (control == null)
		return;

	// hier nicht getLocationOnScreen, weil setLocation
	// relativ zum parent arbeitet, genau wie getLocation
	Point loc = control.getLocation();
	Dimension dim = control.getSize();
	Point newLoc;
	// neue Position entsprechend dem Anker setzen
	switch (anchor) {
		case GridBagConstraints.NORTHWEST :
			newLoc = new Point(loc.x, loc.y);
			break;
		case GridBagConstraints.NORTHEAST :
			newLoc = new Point(loc.x + dim.width, loc.y);
			break;
		case GridBagConstraints.SOUTHEAST :
			newLoc = new Point(loc.x + dim.width, loc.y + dim.height);
			break;
		case GridBagConstraints.SOUTHWEST :
			newLoc = new Point(loc.x, loc.y + dim.height);
			break;
		default :
			newLoc = loc;
	}
	frameMaze.setLocation(newLoc);
}

/**
 * Fenster an die Bildschirmgroesse anpassen.
 * <p>
 * Breite:
 * <br>mindestens 300 Pixel im Inneren
 * <br>Labyrinth komplett anzeigen
 * <br>hoechstens aber bis zum Bildschirmrand
 * <br>Hoehe:
 * <br>Labyrinth komplett anzeigen
 * <br>hoechstens bis zum Bildschirmrand
 *
 * @see ControlFrame#engineAutoResize
 */
public void checkSize() {
	// Position auf dem Bildschirm nicht verfuegbar, wenn
	// das Frame noch nicht angezeigt wird
	if (!frameMaze.isShowing()) return;

	Insets insets = frameMaze.getInsets();
	Point loc = frameMaze.getLocationOnScreen();
	Dimension size = frameMaze.getSize();
	Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
	Rectangle viewrec = scroll.getViewport().getViewRect();

	// viewDX kleiner als 0 = wir haben was uebrig
	// viewDX groesser als 0 = es fehlt was
	// durch Addition von viewDX zur aktuellen Breite
	// kriegen wir genauso viel wie wir brauchen,
	// nicht mehr und nicht weniger
	int viewDX = maze.getWidth()*fs-viewrec.width;
	int viewDY = maze.getHeight()*fs-viewrec.height;
	// linke und rechts Insets des frameMaze
	int insLR = insets.left+insets.right;

	int w, h;
	// mindestens 300 Pixel im Inneren
	w = Math.max(300+insLR, size.width+viewDX);
	// hoechstens aber bis zum Bildschirmrand gehen
	w = Math.min(w, screen.width - loc.x);
	// hoechstens bis zum Bildschirmrand gehen
	h = Math.min(size.height+viewDY, screen.height - loc.y);
	frameMaze.setSize(w, h);
}

/**
 * Ist der Hamster auf den angegebenen Koordinaten?
 * @return boolean
 * @param x int
 * @param y int
 * @param z int
 */
public boolean hamsterAt(int x, int y, int z) {
	return hamster!=null && xpos==x && ypos==y && zpos==z;
}

/**
 * TODO: Erzeugt im vorhandenen Labyrinth ein Zufallslabyrinth.
 * @param wallprob Wahrscheinlichkeit fuer eine Wand (0 bis 100)
 * @param corn Gesamtgroesse des zu verteilenden Mais
 * @param maxheap Maximalgroesse der Haufen (verteilt auf alle
 * Korngroessen)
 */
public void randomFill(int wallprob, int corn, int maxheap) {
	// Alles leer machen
	int[] emptycorn = new int[1+CORNSIZE_MAX];
	for(int i=0; i<maze.levels.size(); i++){
		Level level = maze.getLevel(i);
		for(int x = 0; x<level.getWidth(); x++)
		for(int y = 0; y<level.getHeight(); y++){
			level.removeCellBorder(x,y,BORDER_WEST | BORDER_NORTH);
			level.setCorn(x,y,emptycorn);
		}
	}
	/*
	TODO:
	Labyrinth mit Waenden und Mais fuellen.
	Sollte auch Beamer zwischen den Ebenen setzen,
	falls da noch keine sind. Die vorhandenen Beamer
	koennen ruhig da bleiben.
	*/
	
}

/**
 * Waehlt ein Level zum anzeigen.
 * @param nr int
 */
public void selectLevel(int nr) {
//	System.out.println("selectLevel("+nr+")");
	if (nr<0 || nr>=maze.levels.size())
		return;
	viewLevel = maze.getLevel(viewZ = nr);
	drawMaze();
}

/**
 * Zeichnet die Zelle neu. Schaut selbst, ob der Hamster dort ist.
 * @param x int
 * @param y int
 * @see #updateCell(int,int)
 */
public void updateCell(int x, int y) {
	updateCell(x,y, hamsterAt(x,y,viewZ));
}

/**
 * Zeichnet die Status-Zeile, die dem Editor gehoert.
 */
public void updateEditLine() {
	if (frameMaze == null)
		return;

	String koordString = "(-,-,-) (-,-)", availableString = "-";

	int x = editX;
	int y = editY;

	if (maze.posOK(x,y)){
		availableString = "" + viewLevel.getCorn(x,y,0) + " (";
		for (int gr = 1; gr <= CORNSIZE_MAX; gr++) {
			availableString += viewLevel.getCorn(x,y,gr) + " ";
		}
		availableString += ")";
		koordString = "(" + x + "," + y + "," + viewZ + ")";
		koordString += " ("+(x-viewLevel.centerX)+","+(y-viewLevel.centerY)+")";
	}

	Graphics statI = panelStat.getImageGraphics();
	statI.setColor(RGB_STATUS_BG);
	statI.fillRect(0, 4*sh, 300, sh);

	if (maze.posOK(x,y)){
		statI.setColor(RGB_TEXT);
		statI.drawString("field: " + availableString, 4, 13 + 4*sh);
		statI.drawString(koordString, 4 + 200, 13 + 4*sh);
	}
	panelStat.repaint(0, 4*sh, 300, sh);
}

/**
 * Zeichnet die Statuszeile, die dem Editor gehoert.
 * @param x,y Die Koordinaten des Feldes, auf das die Maus zeigt
 */
public void updateEditLine(int x, int y) {
	if (frameMaze == null)
		return;

	editX = x;
	editY = y;

	updateEditLine();
}

/**
 * 
 * @param title java.lang.String
 */
public void setTitle(String title) {
	if (frameMaze!=null)
		frameMaze.setTitle(title);
}

/**
 * Erzeugt und oeffnet das Labyrinth-Fenster.
 * @param sleepTime Wartezeit pro Schritt
 * @param title Titel des Fensters
 * @param exitAtClose Das Fenster kann geschlossen werden,
 *  und das Programm wird dann beendet
 * @param location gewuenschte Position des Fenster (relativ zum parent!),
 *  wenn sie null ist, positioniert sich das Fenster in Abhaengigkeit von
 *  ControlFrame
 * @param cellSize Pixelgroesse einer Zelle
 */
public void openWindow(int sleepTime, String title,
	boolean exitAtClose, Point location, int cellSize) {

	fs = cellSize;
	viewLevel = maze.getLevel(viewZ = getHomeLevel());
	setSpeed(sleepTime);

	panelStat = new ImagePanel(new Dimension(300, stateHeight));
	panelMaze =
		new ImagePanel(new Dimension(maze.getWidth() * fs, maze.getHeight() * fs));

	createFrame(title, exitAtClose);
	createImages();

	this.control = control;
	if (location==null){
		positionFrame();
	}else
		frameMaze.setLocation(location);

	// drawMaze braucht beide Komponenten komplett fertig
	drawMaze();

	frameMaze.setVisible(true);
	// Groesse erst pruefen, wenn das Fenster angezeigt wird
	checkSize();
}

}