package algds.contest;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Reagiert auf Mausereignisse im panelMaze der EngineX.
 * <br>Creation date: (09.02.01 14:02:54)
 * @author Christian Semrau
 */
public final class EngineXMouse extends MouseInputAdapter
implements MazeConstants, EditorConstants {
	protected ControlFrame control = null;
	protected EngineX engineX = null;
	protected Maze maze = null;
	protected boolean firstDrag;
	protected int lastDragX, lastDragY;

/**
 * Erzeugt ein neues EngineXMouse-Objekt.
 * @param con Das ControlFrame
 * @param eng Die EngineX
 * @throws NullPointerException wenn <code>con==null</code>
 * oder <code>eng==null</code>
 */
public EngineXMouse(ControlFrame con, EngineX eng) {
	super();
	control = con;
	engineX = eng;
	if (con==null)
		throw new NullPointerException("con==null");

	try{
		maze = engineX.maze;
	}catch(NullPointerException exc){
		throw new NullPointerException("eng==null");
	}
}
/**
 * Editiert den Beamer auf dem Feld cx,cy.
 * @param cx,cy Koordinaten des Feldes
 * @param fstButton primaere Maustaste gedrueckt
 */
protected void editBeamer(int cx, int cy, boolean fstButton) {
	Level vl = engineX.viewLevel;
	if (cx<0 || cy<0 || cx>=vl.getWidth() || cy>=vl.getHeight())
		return;

	int index = control.getDetailMode();
	Beamer b = vl.getBeamer(cx, cy);

	if (index == DETAIL_NEWBEAMER) {
		// neuen Beamer anlegen
		if (b == null) {
			// Feld hat noch keinen
			// halben Beamer anlegen
			b = maze.addHalfBeamer(cx, cy, engineX.viewZ);
			engineX.updateCell(cx, cy);
			// zum Connecten auswaehlen
			if (control != null)
				control.selectHalfBeamer(b);
		} else {
			// Feld hat schon einen
			if (b.dest == null)
				// ist ein halber Beamer, also
				// zum Connecten auswaehlen
				if (control != null)
					control.selectHalfBeamer(b);
		}
		return;
	} // if (index==DETAIL_NEWBEAMER

	if (index == DETAIL_DELBEAMER) {
		// Beamer loeschen
		if (b != null) {
			// Feld hat einen
			// Beamer mit Zielbeamer entfernen
			maze.removeBeamer(cx, cy, engineX.viewZ);
			engineX.updateCell(cx, cy);
			if (b.dest != null) {
				// ein Zielbeamer existiert
				if (b.dest.L == engineX.viewZ)
					// und liegt im gleichen Level
					engineX.updateCell(b.dest.X, b.dest.Y);
			} else {
				// kein Zielbeamer
				if (control != null)
					// detailList aktualisieren
					control.selectHalfBeamer(null);
			}
		} // if (b!=null
		return;
	} // if (index==DETAIL_DELBEAMER

	if (index == DETAIL_DISCONNECTBEAMER) {
		// Beamer trennen
		if (b != null) {
			// Feld hat Beamer
			if (b.dest != null) {
				// Feld hat kompletten Beamer
				// Beamer trennen
				Beamer d = maze.disconnectBeamer(b);
				// Felder aktualisieren
				engineX.updateCell(b.X, b.Y);
				if (d.L == engineX.viewZ)
					engineX.updateCell(d.X, d.Y);
				if (control != null)
					// den angeklickten Beamer zum Connecten auswaehlen
					control.selectHalfBeamer(b);

			} // if (b.dest!=null
			else {
				if (control != null)
					// den angeklickten Beamer zum Connecten auswaehlen
					control.selectHalfBeamer(b);
			}
		} // if (b!=null
		return;
	} // if (index==DETAIL_DISCONNECTBEAMER

	if (index >= DETAIL_HBINDEX) {
		Beamer bList = maze.getHalfBeamer(index-DETAIL_HBINDEX);

		// gar kein Beamer zum Verbinden da
		if (bList==null) return;

		if (b==null){
			// Beamer zu einem kompletten erweitern
			b = maze.addBeamer(bList, cx, cy, engineX.viewZ);
			engineX.updateCell(cx,cy);
			if (bList.L == engineX.viewZ)
				engineX.updateCell(bList.X, bList.Y);
			if (control != null)
				// detailList aktualisieren
				control.selectHalfBeamer(null);
		}else{
			// der angeklickte Beamer b hat ein Ziel
			if (b.dest!=null)
				return;
			// derselbe Beamer nochmal!
			if (b.X==bList.X && b.Y==bList.Y && b.L==bList.L)
				return;

			// halbe Beamer koppeln
			maze.connectBeamer(b, bList);

			engineX.updateCell(cx,cy);
			if (bList.L == engineX.viewZ)
				engineX.updateCell(bList.X, bList.Y);
			if (control != null)
				// detailList aktualisieren
				control.selectHalfBeamer(null);
		}
	}
}
/**
 * Editiert den Mais auf dem Feld cx,cy.
 * @param cx,cy Koordinaten des Feldes
 * @param fstButton primaere Maustaste gedrueckt
 */
protected void editCorn(int cx, int cy, boolean fstButton) {
	Level vl = engineX.viewLevel;
	if (cx<0 || cy<0 || cx>=vl.getWidth() || cy>=vl.getHeight())
		return;

	// Gewuenschte Groesse und Anzahl ermitteln
	int index = control.getDetailMode();
	int groesse = index / DETAIL_CORNNUMBER.length + 1;
	int anzahl = DETAIL_CORNNUMBER[index % DETAIL_CORNNUMBER.length];

	//System.out.println("Edit Corn: groesse=" + groesse + ", anzahl=" + anzahl);

	// Wenn kein Fehler auftrat
	if (groesse >= 1 && groesse <= Constants.CORNSIZE_MAX) {
		// links klicken = setzen, rechts klicken = loeschen
		if (fstButton)
			vl.addCorn(cx, cy, groesse, anzahl);
		else
			vl.removeCorn(cx, cy, groesse, anzahl);
		engineX.updateCell(cx, cy);
		engineX.updateEditLine();
	}
}
/**
 * Setzt das Heimatfeld oder das Center des Levels auf das Feld cx,cy.
 * @param cx,cy Koordinaten des Feldes
 * @param fstButton primaere Maustaste gedrueckt
 */
public void editHome(int cx, int cy, boolean fstButton) {
	Level vl = engineX.viewLevel;
	if (cx<0 || cy<0 || cx>=vl.getWidth() || cy>=vl.getHeight())
		return;

	// Gewuenschte Funktion
	int index = control.getDetailMode();

	if (index==DETAIL_SETHOME){
		int hx = maze.homex, hy = maze.homey;
		maze.setHome(cx,cy,engineX.viewZ);
		engineX.updateCell(hx, hy);
		engineX.updateCell(cx, cy);
		return;
	}
	if (index==DETAIL_SETCENTER){
		int hx = vl.centerX, hy = vl.centerY;
		vl.setCenter(cx,cy);
		engineX.updateCell(hx, hy);
		engineX.updateCell(cx, cy);
		return;
	}
}
/**
 * Editiert die Waende auf dem Feld cx,cy.
 * @param cx,cy Koordinaten des Feldes
 * @param mx,my Koordinaten innerhalb des Feldes
 * @param fstButton primaere Maustaste gedrueckt
 */
protected void editWall(int cx, int cy, int mx, int my, boolean fstButton) {

	Level vl = engineX.viewLevel;
	if (cx<0 || cy<0 || cx>=vl.getWidth() || cy>=vl.getHeight())
		return;

	// Seite, deren Wand geaendert werden soll
	int seite = 0;
	if (mx > my) {
		//rechtsoben
		if (mx + my < engineX.fs)
			//linksoben
			seite = NORTH;
		else
			//rechtsunten
			seite = EAST;
	} else {
		//linksunten
		if (mx + my < engineX.fs)
			//linksoben
			seite = WEST;
		else
			//rechtsunten
			seite = SOUTH;
	}

	//			System.out.println("Clicked " + cx + "," + cy + " - "
	//				+ mx + "," + my + " - " + seite);

	// links klicken = setzen, rechts klicken = loeschen
	if (fstButton) {
		vl.addCellBorder(cx, cy, 1 << seite);
	} else {
		vl.removeCellBorder(cx, cy, 1 << seite);
	}

	// Zelle neu zeichnen
	engineX.updateCell(cx, cy);

	// Nachbarzelle neu zeichnen
	int nx = cx + WALK_LUT[seite][WX];
	int ny = cy + WALK_LUT[seite][WY];
	if (maze.posOK(nx, ny))
		engineX.updateCell(nx, ny);
}
/**
 * @see java.awt.event.MouseListener
 */
public void mouseClicked(MouseEvent e) {
	// solange der Hamster laeuft aendern wir nichts!
	if (engineX.running)
		return;

	// Position des Klicks
	int x = e.getX();
	int y = e.getY();
	// Buttons
	int buttons = e.getModifiers();
	// wars die erste Maustaste
	boolean fstButton = (buttons & e.BUTTON1_MASK) != 0;
	// was sollen wir aendern
	int editMode = control.getEditMode();

	// geklicktes Feld
	int cx = x / engineX.fs;
	int cy = y / engineX.fs;

	switch (editMode) {
		case EDIT_WALL :{
			/*
			// Position innerhalb des Feldes
			int mx = x % engineX.fs;
			int my = y % engineX.fs;

			editWall(cx,cy,mx,my,fstButton);
			*/
			break;
		}
		case EDIT_CORN :
			editCorn(cx, cy, fstButton);
			break;
		case EDIT_BEAMER :
			editBeamer(cx, cy, fstButton);
			break;
		case EDIT_HOME :
			editHome(cx, cy, fstButton);
			break;
	}
}
/**
 * @see java.awt.event.MouseListener
 */
public void mouseEntered(MouseEvent e) {
//	System.out.println(e);
//	engineX.panelMaze.addMouseMotionListener(this);
}
/**
 * @see java.awt.event.MouseListener
 */
public void mouseExited(MouseEvent e) {
//	System.out.println(e);
//	engineX.panelMaze.removeMouseMotionListener(this);
	engineX.updateEditLine(-1,-1);
}
/**
 * @see java.awt.event.MouseMotionListener
 */
public void mouseMoved(MouseEvent e) {
//	System.out.println(e);
	// Position des Klicks
	int x = e.getX();
	int y = e.getY();
	int cx = x/engineX.fs;
	int cy = y/engineX.fs;
	if (cx!=engineX.editX || cy!=engineX.editY)
		engineX.updateEditLine(cx,cy);
}

/**
 * 
 * @param e java.awt.event.MouseEvent
 */
public void mouseDragged(MouseEvent e) {
	// solange der Hamster laeuft aendern wir nichts!
	if (engineX.running)
		return;

	// Position des Klicks
	int x = e.getX();
	int y = e.getY();
	// Buttons
	int buttons = e.getModifiers();
	// wars die erste Maustaste
	boolean fstButton = (buttons & e.BUTTON1_MASK) != 0;
	// was sollen wir aendern
	int editMode = control.getEditMode();

	// geklicktes Feld
	int cx = x / engineX.fs;
	int cy = y / engineX.fs;
	int lcx = lastDragX / engineX.fs;
	int lcy = lastDragY / engineX.fs;

	switch (editMode) {
		case EDIT_WALL :{
			/*
			// Position innerhalb des Feldes
			int mx = x % engineX.fs;
			int my = y % engineX.fs;

			editWall(cx,cy,mx,my,fstButton);
			*/
			break;
		}
		case EDIT_CORN :
			if (firstDrag || lcx!=cx || lcy!=cy)
				editCorn(cx, cy, fstButton);
			break;
		case EDIT_BEAMER :
			/*
			editBeamer(cx, cy, fstButton);
			*/
			break;
		case EDIT_HOME :
			if (firstDrag || lcx!=cx || lcy!=cy)
				editHome(cx, cy, fstButton);
			break;
	}
	lastDragX = x;
	lastDragY = y;
	firstDrag = false;
}

/**
 * 
 * @param e java.awt.event.MouseEvent
 */
public void mousePressed(MouseEvent e) {
	lastDragX = e.getX();
	lastDragY = e.getY();
	firstDrag = true;

	// solange der Hamster laeuft aendern wir nichts!
	if (engineX.running)
		return;

	// Position des Klicks
	int x = e.getX();
	int y = e.getY();
	// Buttons
	int buttons = e.getModifiers();
	// wars die erste Maustaste
	boolean fstButton = (buttons & e.BUTTON1_MASK) != 0;
	// was sollen wir aendern
	int editMode = control.getEditMode();

	// geklicktes Feld
	int cx = x / engineX.fs;
	int cy = y / engineX.fs;

	switch (editMode) {
		case EDIT_WALL :{
			// Position innerhalb des Feldes
			int mx = x % engineX.fs;
			int my = y % engineX.fs;

			editWall(cx,cy,mx,my,fstButton);
			break;
		}
	}
}
}