package algds.contest;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * Laedt JAVA-Klassen aus einen anzugebenden Verzeichnis.
 * Stammt aus den Sun JAVA-Tutorien.
 * Wird hier genutzt, um die Hamster neu zu laden.
 * @see Utils#loadHamster(java.lang.String, algds.contest.Logable)
 */
public class FileClassLoader extends ClassLoader {

  private String root;

  public FileClassLoader (String rootDir) {
	if (rootDir == null)
	  throw new IllegalArgumentException ("Null root directory");
	root = rootDir;
  }  

  protected Class loadClass (String name, boolean resolve) 
	throws ClassNotFoundException {

	// System.out.println("Loader: Lade "+name);

	Class c = findLoadedClass (name);
	// if (c!=null) System.out.println("Loader: "+name+" bereits da.");

	if (c == null) {
	  // Convert class name argument to filename
	  // Convert package names into subdirectories
	  String filename = name.replace ('.', File.separatorChar) + ".class";

	  try {
		byte data[] = loadClassData(filename);
		c = defineClass (name, data, 0, data.length);
		// System.out.println("Loader: "+name+" selbst geladen.");
	  } catch (IOException e) {

	  }
	}

	// Since all support classes of loaded class use same class loader
	// must check subclass cache of classes for things like Object
	if (c == null) {
	  try {
		c = findSystemClass (name);
		// if (c!=null) System.out.println("Loader: "+name+" als Systemklasse geladen.");
	  } catch (Exception e) {
	  }
	}

	if (c == null)
	  throw new ClassNotFoundException (name);

	if (resolve)
	  resolveClass (c);
	return c;
  }  

  private byte[] loadClassData (String filename) 
	  throws IOException {

	// Create a file object relative to directory provided
	File f = new File (root, filename);

	// Get size of class file
	int size = (int)f.length();

	// Reserve space to read
	byte buff[] = new byte[size];

	// Get stream to read from
	FileInputStream fis = new FileInputStream(f);
	DataInputStream dis = new DataInputStream (fis);

	// Read in data
	dis.readFully (buff);

	// close stream
	dis.close();

	// return data
	return buff;
  }  
}