package algds.contest;

/**
 * Dieses Interface enthaelt Konstanten fuer die interne Verarbeitung
 * der Labyrinthe.
 * <br>Creation date: (05.02.01 22:41:14)
 * @author Johannes Zander, Christian Semrau
 */
public interface MazeConstants {

	/** Interne Richtungsangaben. Duerfen nicht veraendert werden!!
	 * Werden unter anderem benutzt, um die Labyrinthe zu laden und
	 * zu speichern.
	 */
	static final int
		WEST = 0, NORTH = 1, EAST = 2, SOUTH = 3;

		
	static final int[] antidir = new int[]
		{EAST, SOUTH, WEST, NORTH};

	/**
	 * Werte der Waende, wenn sie bitweise verknuepft werden.
	 * @see Level#border
	 */
	static final int
		BORDER_WEST  = 1 << WEST,
		BORDER_NORTH = 1 << NORTH,
		BORDER_EAST  = 1 << EAST,
		BORDER_SOUTH = 1 << SOUTH;

	/**
	 * Konstanten, die den Besuchsstatus der Felder des Labyrinths
	 * angeben.
	 * @see Level#visField
	 */
	final static int
		SAW_WEST  = 1 << 0,
		SAW_NORTH = 1 << 1,
		SAW_EAST  = 1 << 2,
		SAW_SOUTH = 1 << 3,
		SAW_FIELD = 1 << 4,
		VIS_FIELD = 1 << 5,
		SAW_ALL = SAW_WEST|SAW_NORTH|SAW_EAST|SAW_SOUTH|SAW_FIELD|VIS_FIELD;

	/** Wandelt die interne Richtung in die Hamster-Richtung um. */
	static final int[] DIR_LUT = new int[]{
		Constants.DIR_WEST,
		Constants.DIR_NORTH,
		Constants.DIR_EAST,
		Constants.DIR_SOUTH};

	/** Ortsaenderung bei einem Schritt. [Richtung][Achse] */
	static final int[][] WALK_LUT = new int[][]{
		{-1, 0}, {0, -1}, {1, 0},{0, 1}
	};

	/** Angabe der Koordinatenachse fuer WALK_LUT. */
	static final int WX = 0, WY = 1;
}