package algds.contest;

import java.io.*;
import java.util.*;
import java.util.zip.*;

/**
 * Laedt und speichert Labyrinthe.
 * Um ein Labyrinth zu laden, konstruiert man ein MazeIO-Objekt und
 * ruft die load-Methode auf:
 * <br><code>Maze maze = new MazeIO().load(dateiname);<code>
 * <p>
 * Um ein Labyrinth zu speichern, konstruiert man ein MazeIO-Objekt und
 * ruft die save-Methode auf:
 * <br><code>new MazeIO().save(maze,dateiname);<code>
 * <p>
 * @author Johannes Zander, Christian Semrau
 */
public final class MazeIO implements MazeConstants{

/**
 * Laedt ein Labyrinth des Formats 2000 oder 2001.
 *
 * @return algds.contest.Maze
 * @param filename java.lang.String
 * @author Johannes Zander, Christian Semrau
 * @throws IOException wenn beim Laden der Datei ein Fehler auftrat
 */
public Maze load(String name) throws IOException {
	maze = null;

	// eventuell Endung anhaengen
	String name2 = addExtension(name);

	// komplette Datei einlesen
	if (name2.toLowerCase().endsWith(".zip")){

		zeile = Utils.readZipEntries(name2);
		System.out.println("ZipFile: "+name2);
		for(int i=0; i<zeile.length; i++)
			System.out.println("File: "+zeile[i]);
		zeile = Utils.readZipStringArray(name2,zeile[0]);
	}else
		zeile = Utils.readStringArray(name2,true);

	// bei Fehlern wird null als Feld geliefert
	if (zeile == null)
		throw new IOException("Konnte Datei nicht laden.");

	// zerlegt die Zeilen
	StringTokenizer st;

	// 0 = Typ von 2000
	// 1 = Typ von 2001
	int saveType = 0;

	// index der gerade bearbeiteten Zeile
	index = -1;

	// es koennen uns einige Exceptions passieren,
	// wenn die Datei fehlerhaft ist,
	// abgefangen und in IOException umgewandelt werden:
	// NumberFormatException
	// NoSuchElementException
	// ArrayIndexOutOfBoundException
	try {

		// testen, welchen Speicher-Typ wir haben
		if (parseToken2(zeile[index+1], "maze2001")){
			saveType = 1;
			++index;
		}

		String author = "";

		if (saveType==0){
			levelCount = parseToken(zeile[++index], "mazeLevels");
			width = parseToken(zeile[++index], "width");
			height = parseToken(zeile[++index], "heigth"); // heigTH
			cornsizemax = 1;
		}else{
			if (parseToken2(zeile[index+1], "author")){
				++index;
				author = zeile[++index];
			}

			levelCount = parseToken(zeile[++index], "mazeLevels");
			width = parseToken(zeile[++index], "width");
			height = parseToken(zeile[++index], "height"); // heigHT
			parseToken2(zeile[++index], "constants");
			st = new StringTokenizer(zeile[++index], ",");
			cornsizemax = Integer.parseInt(st.nextToken());
		}

		// Groesse der Level kennen wir nun
		maze = new Maze(width, height);

		// Autor setzen
		maze.author = author;

		// Waende und Koerner der Level laden
		for (int l = 0; l < levelCount; l++) {
			// die Level werden in aufsteigender Reihenfolge gespeichert
			int li = parseToken(zeile[++index], "level");
			if (li != l)
				throw new IOException("Falscher Level-Index "+li+" (muss "+l+" sein) in Zeile "+index);

			// ein Level laden
			/*
			Die moegliche IllegalArgumentException koennen
			wir ignorieren, wenn sichergestellt ist, dass
			die Level die gleiche Groesse haben wie das
			Labyrinth.
			*/
			if (saveType==0)
				maze.addLevel(loadLevel2000());
			else
				maze.addLevel(loadLevel2001());
		}

		// Beamer-Zaehler zuruecksetzen
		Beamer.counter = 0;

		// komplette Beamer laden
		int beamerCount = parseToken(zeile[++index], "beamers");
		for (int bc = 0; bc < beamerCount; bc++) {
			st = new StringTokenizer(zeile[++index], ",");

			int firstLevel = Integer.parseInt(st.nextToken());
			int firstX = Integer.parseInt(st.nextToken());
			int firstY = Integer.parseInt(st.nextToken());
			Beamer beam = new Beamer(firstX, firstY, firstLevel);

			int secondLevel = Integer.parseInt(st.nextToken());
			int secondX = Integer.parseInt(st.nextToken());
			int secondY = Integer.parseInt(st.nextToken());
			new Beamer(secondX, secondY, secondLevel, beam);

			// beide haelften einfuegen
			maze.addBeamer(beam);
		}

		// halbe Beamer laden
		int halfBeamerCount = parseToken(zeile[++index], "halfbeamers");
		for (int hbc = 0; hbc < halfBeamerCount; hbc++) {
			st = new StringTokenizer(zeile[++index], ",");

			int firstLevel = Integer.parseInt(st.nextToken());
			int firstX = Integer.parseInt(st.nextToken());
			int firstY = Integer.parseInt(st.nextToken());
			Beamer beam = new Beamer(firstX, firstY, firstLevel);

			maze.addBeamer(beam);
		}
		if (saveType==1){
			parseToken2(zeile[++index], "home");
			st = new StringTokenizer(zeile[++index], ",");
			int hx = Integer.parseInt(st.nextToken());
			int hy = Integer.parseInt(st.nextToken());
			int hz = Integer.parseInt(st.nextToken());
			maze.setHome(hx, hy, hz);
			parseToken2(zeile[++index], "centers");
			for (int l = 0; l < levelCount; l++) {
				Level level = maze.getLevel(l);
				st = new StringTokenizer(zeile[++index], ",");
				int cx = Integer.parseInt(st.nextToken());
				int cy = Integer.parseInt(st.nextToken());
				level.centerX = cx;
				level.centerY = cy;
			}
		}

		// validate + fix
		for (int l = 0; l < levelCount; l++)
			for (int h = 0; h < height; h++)
				for (int w = 0; w < width; w++) {
					Cell cellLeft = (w == 0) ? null : maze.getCell(w - 1, h, l);
					Cell cellUp = (h == 0) ? null : maze.getCell(w, h - 1, l);
					Cell cell = maze.getCell(w, h, l);
					// beachte: isBorder(EAST), aber addCellBorder(BORDER_EAST) !
					if (w > 0 && (cellLeft.isBorder(EAST) != cell.isBorder(WEST))) {
						cellLeft.addCellBorder(BORDER_EAST);
						cell.addCellBorder(BORDER_WEST);
					}

					if (h > 0 && (cellUp.isBorder(SOUTH) != cell.isBorder(NORTH))) {
						cellUp.addCellBorder(BORDER_SOUTH);
						cell.addCellBorder(BORDER_NORTH);
					}
				}

		// safety borders
		for (int l = 0; l < levelCount; l++) {
			Level level = maze.getLevel(l);
			for (int h = 0; h < height; h++) {
				level.addCellBorder(0, h, BORDER_WEST);
				level.addCellBorder(width - 1, h, BORDER_EAST);
			}

			for (int w = 0; w < width; w++) {
				level.addCellBorder(w, 0, BORDER_NORTH);
				level.addCellBorder(w, height - 1, BORDER_SOUTH);
			}
		}

	} catch (NumberFormatException exc) {
		throw new IOException("Keine Zahl in Zeile " + index+
			(index<zeile.length?("\n"+zeile[index]):""));
	} catch (ArrayIndexOutOfBoundsException exc) {
		throw new IOException("Unerwartetes Ende in Zeile " + index+
			(index<zeile.length?("\n"+zeile[index]):""));
	} catch (NoSuchElementException exc) {
		throw new IOException("Unerwartetes Ende in Zeile " + index+
			(index<zeile.length?("\n"+zeile[index]):""));
	} catch (NullPointerException exc) {
		throw new IOException("Interner Fehler: NullPointerException in Zeile " + index+
			(index<zeile.length?("\n"+zeile[index]):""));
	}
	return maze;
}

/**
 * Filtert aus einem String eine Zahl.
 * Der String <code>stream</code> hat das Format
 * <code>[token:zahl]</code>. Die zahl wird geliefert.
 * @throws NoSuchElementException
 * wenn das Token nicht gefunden wurde.
 * @throws NumberFormatException
 * wenn danach keine Zahl folgt.
 */
private static int parseToken(String stream, String token) {
	StringTokenizer st = new StringTokenizer(stream, " []:");

	if (st.nextToken().equals(token))
		return Integer.parseInt(st.nextToken());
	else
		throw new NoSuchElementException("Token '" + token + "' not found!");
}

/**
 * Speichert ein Labyrinth im Speicherformat 2001.
 */
public void save(Maze maze, String name) throws IOException {
	String n = Utils.lineSeparator;
	String name2 = addExtension(name);

	Vector zeilen = new Vector();

	levelCount = maze.getLevelCount();
	width = maze.getWidth();
	height = maze.getHeight();

	// neuer Speichertyp
	zeilen.add("[maze2001]" + n);

	if (maze.author!=null && maze.author.length()>0){
		zeilen.add("[author]" + n);
		zeilen.add(maze.author + n);
	}

	zeilen.add("[mazeLevels:" + levelCount + "]" + n);
	zeilen.add("[width:" + width + "]" + n);
	zeilen.add("[height:" + height + "]" + n); // heigHT
	zeilen.add("[constants]" + n);
	zeilen.add(Hamster.CORNSIZE_MAX + "," + n);

	for (int l = 0; l < levelCount; l++) {
		zeilen.add("[level:" + l + "]" + n);
		Level lev = maze.getLevel(l);
		for (int h = 0; h < height; h++) {
			String s = "";
			for (int w = 0; w < width; w++) {
				Cell c = lev.getCell(w, h);
				// fuer jedes Feld die Waende und die Koerner speichern
				s += c.getBorders() + ",";
				int[] corn = c.getCorn();
				for(int gr=1; gr<=Hamster.CORNSIZE_MAX; gr++)
					s += corn[gr] + ",";
			}
			zeilen.add(s+n);
		}
	}
	// zwei Beamer-Eintraege in maze.beamers bilden zusammen
	// einen kompletten Beamer
	int beamercount = (maze.getBeamerCount()/2);
	zeilen.add("[beamers:" + beamercount + "]" + n);

	for (int b = 0; b < maze.getBeamerCount(); b++) {
		Beamer beam = maze.getBeamer(b);
		if (beam.primary)
		zeilen.add(beam.L+","+beam.X+","+beam.Y+","
			+beam.dest.L+","+beam.dest.X+","+beam.dest.Y+","+n);
	}

	zeilen.add("[halfbeamers:" + maze.getHalfBeamerCount() + "]" + n);
	for (int hb = 0; hb < maze.getHalfBeamerCount(); hb++) {
		Beamer beam = maze.getHalfBeamer(hb);
		zeilen.add(beam.L + "," + beam.X + "," + beam.Y + "," + n);
	}

	// reale Heimatkoordinaten
	zeilen.add("[home]"+n);
	zeilen.add(maze.getHomeX()+","+maze.getHomeY()+","+maze.getHomeLevel()+","+n);

	// reale Koordinaten der logischen (0,0)-Felder
	zeilen.add("[centers]"+n);
	for (int l = 0; l < levelCount; l++) {
		Level level = maze.getLevel(l);
		zeilen.add(level.centerX+","+level.centerY+","+n);
	}

	if (!name2.toLowerCase().endsWith(".zip")){
		Utils.writeVector(name2, zeilen);
	}else{
		String entryname = new File(name).getName();
		// und jetzt das ".zip" durch ".maze" ersetzen
		entryname = entryname.substring(0, entryname.length()-4)+".maze";
		Utils.writeVectorZip(name2, entryname, zeilen);
	}

}

	/** Fuer das zu ladende Labyrinth verwendeter Wert von CORNSIZE_MAX */
	int cornsizemax;
	int height;
	int index;
	/** Anzahl der Level. */
	int levelCount;
	Maze maze;
	int width;
	String[] zeile;

/**
 * 
 */
public MazeIO() {}

/**
 * 
 * @return java.lang.String
 * @param s java.lang.String
 */
public static String addExtension(String name) {
	if (name==null) return null;
	if (name.length()<=0) return "";
	return (name.toLowerCase().endsWith(".maze") ? name :
		name.toUpperCase().endsWith(".MAZ") ? name :
		name.toLowerCase().endsWith(".zip") ? name :
		 name + ".maze");
}

/**
 * Laedt ein Level des Speichertyps vom Jahr 2000.
 * @return algds.contest.Level
 */
public Level loadLevel2000() {
	Level lev = new Level(width, height);
	StringTokenizer st;
	for (int h = 0; h < height; h++) {
		st = new StringTokenizer(zeile[++index], ",");

		for (int w = 0; w < width; w++) {
			int border = Integer.parseInt(st.nextToken());
			lev.addCellBorder(w, h, border);
			int beans = Integer.parseInt(st.nextToken());
			lev.addCorn(w, h, 1, beans);
		}
	}
	return lev;
}

/**
 * Laedt ein Level des Speichertyps vom Jahr 2001.
 * @return algds.contest.Level
 */
public Level loadLevel2001() {
	Level lev = new Level(width, height);
	StringTokenizer st;
	for (int h = 0; h < height; h++) {
		st = new StringTokenizer(zeile[++index], ",");

		for (int w = 0; w < width; w++) {
			int border = Integer.parseInt(st.nextToken());
			lev.addCellBorder(w, h, border);
			for(int gr = 1; gr<=cornsizemax; gr++){
				int anz = Integer.parseInt(st.nextToken());
				lev.addCorn(w, h, gr, anz);
			}
		}
	}
	return lev;
}

/**
 * Testroutine. Konvertiert ein Labyrinth in das neue Format.
 * @param args java.lang.String[]
 */
public static void main(String[] args) {
	if (args.length!=2){
		System.out.println("Konvertiert ein Labyrinth des 2000er Formats");
		System.out.println("in eins des 2001er Formats.");
		System.out.println("<input> und <output> duerfen gleich sein.");
		System.out.println("(c) 2001 Christian Semrau, Stephan Finn");
		System.out.println("MazeIO <input> <output>");
		return;
	}
	try{
		Maze maze = new MazeIO().load(args[0]);
		new MazeIO().save(maze, args[1]);
	}catch(IOException exc){
		System.out.println(exc.getMessage());
	}
}

/**
 * Liefert true, wenn der String <code>stream</code> das Token
 * <code>token</code> als erstes enthaelt, also das Format
 * <code>[token]</code> oder <code>[token:zahl]</code> hat.
 */
private static boolean parseToken2(String stream, String token) {
	StringTokenizer st = new StringTokenizer(stream, " []:");
	return st.nextToken().equals(token);
}
}