package algds.contest;
import java.io.*;
import java.util.Date;

/**
 * Grafikloser Lauf des Hamsters.
 * @author Kai-Uwe Sattler, Christian Semrau, Stephan Finn
 */
public final class MazeRunner {
	/** Verzeichnis, in dem das Labyrinth gesucht wird. */
	static String mazeDir = "mazes" + java.io.File.separator;

	static int points, steps, turns, energy, collisions, corn, ate;

/**
 * Laedt Labyrinth und Hamster, und laesst den Hamster laufen.
 * @param String[] [0]=Maze, [1]=Hamster
 */
public static void main(String[] args) {
	int res = work(args);
	System.exit(res);
} // main

/**
 * Laedt Labyrinth und Hamster, und laesst den Hamster laufen.
 * @param args String[] [0]=Maze, [1]=Hamster
 */
public static int work(String[] args) {

	if (args.length < 2) {
		System.out.println(
			"usage: java algds.contest.MazeRunner " +
			"<Maze> <Hamster> <[v][m][f OutputFile]>");
		System.out.println(
			"v = verbose, m = mute, f = save result to file");
		return(1);
	}
	String mazeName = MazeIO.addExtension(args[0]);
	Maze maze = null;
	Hamster hamster = null;

	boolean makeVerbose = false;
	boolean makeMute = false;
	boolean fileoutput = false;

	if (args.length>2){
		String opt = args[2].toLowerCase();
		if (opt.indexOf("v")>=0) makeVerbose=true;
		if (opt.indexOf("m")>=0) makeMute=true;
		if (opt.indexOf("f")>=0) fileoutput=true;
		if (args.length<=3)
			fileoutput=false;
	}

	// Beim Programmstart die Properties laden
	PropIO.load(Logable.NUL_LOG);

	try {
		maze = new MazeIO().load(mazeDir + mazeName);
	} catch (Exception e) {
		System.out.println("cannot load maze: " + e.toString());
		return(1);
	}

	try {
		hamster = (Hamster) Class.forName(args[1]).newInstance();
	} catch (Exception e) {
		System.out.println("cannot load hamster: " + e.toString());
		return(1);
	}

	MazeEngineImpl engine = new MazeEngineImpl(maze);

	engine.setHamster(hamster);
	engine.verbose = makeVerbose;
	engine.mute = makeMute;

	// MazeEngineImpl schlaeft nicht,
	// das tut nur EngineX

	long startZeit = (new Date()).getTime();

	// startet den Hamster
	Object lock = new Object();
	engine.runHamster(lock);

	// wartet bis der Hamster fertig ist
	synchronized (lock) {
		try {
			lock.wait();
		} catch (InterruptedException exc) {
		}
	}

	long endZeit = (new Date()).getTime();

	points = engine.getPoints();
	steps = engine.getSteps();
	collisions = engine.getCollisions();
	energy = engine.getEnergy();
	turns = engine.getTurns();
	corn = engine.getCollectedCorn(0);
	ate = engine.getAte();

	System.out.println(
		"Points: " + points +
		", Steps: " + steps +
		", Turns: " + turns +
		", Corn: " + corn +
		", Coll: " + collisions +
		", Energy: " + energy +
		", Ate: " + ate +
		", Time: " + (endZeit-startZeit)/1000);

	if (fileoutput){
	try{
		// Ausgabe an args[3] anhaengen
		BufferedWriter bw = new BufferedWriter(new FileWriter(args[3], true));
		bw.write(
			"Hamster: "+args[1]+
			", Maze: "+mazeName+
			" / Points: " + points +
			", Steps: " + steps +
			", Turns: " + turns +
			", Corn: " + corn +
			", Coll: " + collisions +
			", Energy: " + energy +
			", Ate: " + ate +
			", Time: " + (endZeit-startZeit)/1000);
		if (engine.exceptionString!=null){
			// engine.exceptionString hat bereits ein "\n" davor
			// und dahinter
			bw.write("\nException: {"+engine.exceptionString);
			if (engine.hamsterException!=null){
				engine.hamsterException.printStackTrace(new PrintWriter(bw));
			}
			bw.write("}");
		}
		bw.newLine();
		bw.close();
	}catch(Exception exc){
		System.out.println("Exception occured while saving results:\n"+exc);
	}
	}
	return(0);
}
}