package algds.contest;
import java.io.*;
import javax.swing.*;
import java.util.Date;

/**
 * Grafischer Lauf des Hamsters.
 * <br>Stammt original aus dem
 * <br>MazeRunnerX v1.0 by Johannes Zander (io@wtal.de)
 *
 * @author Johannes Zander, Christian Semrau, Stephan Finn
 */
final public class MazeRunnerX2 {
	/** Verzeichnis, in dem das Labyrinthe gesucht wird. */
	static String mazeDir = "mazes" + java.io.File.separator;

	static int points, steps, turns, energy, collisions, corn, ate;

/**
 * Laedt Labyrinth und Hamster, und laesst den Hamster laufen.
 * @param String[] [0]=Maze, [1]=Hamster
 */
public static void main(String[] args) {

	if (args.length < 3 || args.length > 5) {
		System.out.println(
			"usage: java algds.contest.MazeRunnerX2" +
			" <Maze> <Hamster1> <Hamster2> <[sleepTime][s][v][m][f outputFile]>");
		System.out.println(
			"s = toggle SloMo, v = verbose, m = mute, f = save result to file");
		return;
	}

	final String[] a1, a2;
	if (args.length==3){
	  a1 = new String[]{args[0], args[1]};
	  a2 = new String[]{args[0], args[2]};
	}else
	if (args.length==4){
	  a1 = new String[]{args[0], args[1], args[3]};
	  a2 = new String[]{args[0], args[2], args[3]};
	}else{
	  a1 = new String[]{args[0], args[1], args[3], args[4]};
	  a2 = new String[]{args[0], args[2], args[3], args[4]};
	}

	new Thread(){
	  public void run(){
	    work(a1);
	  }
	}.start();
	new Thread(){
	  public void run(){
            work(a2);
	  }
	}.start();
}

/**
 * Laedt Labyrinth und Hamster, und laesst den Hamster laufen.
 * @param args String[] [0]=Maze, [1]=Hamster
 */
public static int work(String args[]) {

	if (args.length < 2 || args.length > 4) {
		System.out.println(
			"usage: java algds.contest.MazeRunnerX" +
			" <Maze> <Hamster> <[sleepTime][s][v][m][f outputFile]>");
		System.out.println(
			"s = toggle SloMo, v = verbose, m = mute, f = save result to file");
		return(1);
	}
	String mazeName = MazeIO.addExtension(args[0]);
	Maze maze = null;
	Hamster hamster = null;

	boolean makeVerbose = false;
	boolean makeMute = false;
	boolean fileoutput = false;
	boolean toggleSlomo = false;
	int sleepTime = ControlFrame.defaultSleep;

	if (args.length>2){
		String opt = args[2].toLowerCase();
		int pv, pm, pf, ps;
		if ((ps=opt.indexOf("s"))>=0) toggleSlomo=true; else ps=opt.length();
		if ((pv=opt.indexOf("v"))>=0) makeVerbose=true; else pv=opt.length();
		if ((pm=opt.indexOf("m"))>=0) makeMute=true; else pm=opt.length();
		if ((pf=opt.indexOf("f"))>=0) fileoutput=true; else pf=opt.length();
		int min = Math.min(Math.min(pv, pm), Math.min(pf, ps));
		try{
			sleepTime = Integer.parseInt(opt.substring(0,min));
		}catch(NumberFormatException exc){
		}
		if (args.length<=3)
			fileoutput=false;
	}

	// Beim Programmstart die Properties laden
	PropIO.load(Logable.NUL_LOG);
	
	if (toggleSlomo) EngineX.SLO_MO = !EngineX.SLO_MO;

	try {
		maze = new MazeIO().load(mazeDir + mazeName);
		System.out.println(mazeDir + mazeName+" loaded.");
	} catch (Exception e) {
		System.out.println("cannot load maze: " + e.toString());
		return(1);
	}

	try {
		hamster = (Hamster) Class.forName(args[1]).newInstance();
		System.out.println(hamster.getName()+" loaded.");
	} catch (Exception e) {
		System.out.println("cannot load hamster: " + e.toString());
		return(1);
	}


	try {
		UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} catch (Exception e) {
		e.printStackTrace(System.out);
	}

	String title = hamster.getName() + " in " + mazeName;
	EngineX engine = new EngineX(maze, null);
	engine.openWindow(sleepTime, title+" running", true, null, 16);

	engine.setHamster(hamster);
	engine.verbose = makeVerbose;
	engine.mute = makeMute;

	// vor dem Start kurz warten
	// in der Zeit kann das Fenster angezeigt werden
	try {
		Thread.sleep(1000);
	} catch (InterruptedException exc) {
	}

	long startZeit = (new Date()).getTime();

	// startet den Hamster
	Object lock = new Object();
	engine.runHamster(lock);

	// wartet bis der Hamster fertig ist
	synchronized (lock) {
		try {
			lock.wait();
		} catch (InterruptedException exc) {
		}
	}

	long endZeit = (new Date()).getTime();

	// Hamster ist nicht mehr "running"
	engine.setTitle(title);

	points = engine.getPoints();
	steps = engine.getSteps();
	collisions = engine.getCollisions();
	energy = engine.getEnergy();
	turns = engine.getTurns();
	corn = engine.getCollectedCorn(0);
	ate = engine.getAte();


	System.out.println(
		"Points: " + points +
		", Steps: " + steps +
		", Turns: " + turns +
		", Corn: " + corn +
		", Collisions: " + collisions +
		", Energy: " + energy +
		", Ate: " + ate +
		", Time: " + (endZeit-startZeit)/1000);

	if (fileoutput){
	try{
		// Ausgabe an args[3] anhaengen
		BufferedWriter bw = new BufferedWriter(new FileWriter(args[3], true));
		bw.write(
			"Hamster: "+args[1]+
			", Maze: "+mazeName+
			" / Points: " + points +
			", Steps: " + steps +
			", Turns: " + turns +
			", Corn: " + corn +
			", Collisions: " + collisions +
			", Energy: " + energy +
			", Ate: " + ate +
			", Time: " + (endZeit-startZeit)/1000);
		if (engine.exceptionString!=null){
			// engine.exceptionString hat bereits ein "\n" davor
			// und dahinter
			bw.write("\nException: {"+engine.exceptionString);
			if (engine.hamsterException!=null){
				engine.hamsterException.printStackTrace(new PrintWriter(bw));
			}
			bw.write("}");
		}
		bw.newLine();
		bw.close();
	}catch(Exception exc){
		System.out.println("Exception occured while saving results:\n"+exc);
	}
	}
	return(0);
}
}