package algds.contest;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;

/**
 * Dialogfenster, das einige Eigenschaften setzt.
 *
 * <br>Creation date: (12.02.01 22:50:26)
 * @author Christian Semrau, Stephan Finn
 */
public class PropertiesDialog extends JFrame
implements ActionListener, ChangeListener, Logable{
	private JTextField textCreator;
	private JTextField textAuthor;
	private Checkbox checkboxResize;
	private Checkbox checkboxDocking;
	private Checkbox checkboxSLOMO;
	private Checkbox checkboxHamster;
	private Checkbox checkboxSystemDialogs;
	private Checkbox checkboxStandardLoader;
	private JSlider sliderSleep;
	private JSlider sliderSize;
	private JButton buttonOK;
	private JButton buttonCancel;
	private JButton buttonLoad;
	private JButton buttonSave;
	private JLabel labelSleep;
	private JLabel labelSize;

	private PropertiesIO IO;
	private String author;
	private ControlFrame cf;

/**
 * Erzeugt einen Dialog mit uebergebenen Werten.
 * @param author null, wenn kein Labyrinth geladen ist.
 */
public PropertiesDialog(String title, ControlFrame CF) {
	super(title);
	cf = CF;

	IO = new PropertiesIO(
	ControlFrame.creatorName,
	ControlFrame.engineAutoResize,
	ControlFrame.engineDocking,
	EngineX.SLO_MO,
	EngineX.showHamsterImg,
	ControlFrame.systemDialogs,
	ControlFrame.standardLoader,
	ControlFrame.defaultSleep,
	ControlFrame.defaultCellSize);
	IO.log = this;

	author = (cf==null?null:(cf.maze!=null?cf.maze.author:null));

	createComponents();
	constructGUI();
}

private void createComponents() {
	textCreator = new JTextField(IO.creator, 20);
	textAuthor = new JTextField(author, 20);
	if (author==null) textAuthor.setEnabled(false);
	checkboxResize = new Checkbox("AutoResize", IO.resize);
	checkboxDocking = new Checkbox("Docking", IO.docking);
	checkboxSLOMO = new Checkbox("SloMo", IO.slomo);
	checkboxHamster = new Checkbox("Hamster Image", IO.hamsterimg);
	checkboxSystemDialogs = new Checkbox("System Dialogs", IO.systemdialogs);
	checkboxStandardLoader = new Checkbox("StandardLoader", IO.standardloader);
	if (IO.sleep<50) IO.sleep=50;
	if (IO.sleep>990) IO.sleep = 990;
	if (IO.size<16) IO.size=16;
	if (IO.size>32) IO.size=32;
	sliderSleep = new JSlider(5,99,IO.sleep/10);
	sliderSize = new JSlider(16,32,IO.size);

	labelSleep = new JLabel("Sleep: "+(IO.sleep));
	labelSize = new JLabel("Size: "+IO.size);
	buttonOK = new JButton("OK");
	buttonCancel = new JButton("Cancel");
	buttonLoad = new JButton("Load");
	buttonSave = new JButton("Save");
}

private void constructGUI(){
	addWindowListener(new WindowAdapter(){
		public void windowClosing(WindowEvent ev){
			dispose();
			cancelPressed();
		}
	});
	buttonOK.addActionListener(this);
	buttonCancel.addActionListener(this);
	buttonLoad.addActionListener(this);
	buttonSave.addActionListener(this);

	sliderSleep.addChangeListener(this);
	sliderSize.addChangeListener(this);

	Container pane = getContentPane();
	pane.setLayout(new GridBagLayout());

	int sy = 0;

	pane.add(new Label("Creator"), new GridBagConstraints(0, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(textCreator, new GridBagConstraints(1, sy, 3, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	sy++;

	pane.add(new Label("Author"), new GridBagConstraints(0, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(textAuthor, new GridBagConstraints(1, sy, 3, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	sy++;

	pane.add(checkboxSLOMO, new GridBagConstraints(0, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(checkboxResize, new GridBagConstraints(1, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(checkboxDocking, new GridBagConstraints(2, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(checkboxHamster, new GridBagConstraints(3, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	sy++;

	pane.add(checkboxSystemDialogs, new GridBagConstraints(0, sy, 2, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(checkboxStandardLoader, new GridBagConstraints(2, sy, 2, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	sy++;

	pane.add(labelSleep, new GridBagConstraints(0, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(sliderSleep, new GridBagConstraints(1, sy, 3, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	sy++;

	pane.add(labelSize, new GridBagConstraints(0, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(sliderSize, new GridBagConstraints(1, sy, 3, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 0), 0, 0));
	sy++;

	pane.add(buttonOK, new GridBagConstraints(0, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(buttonCancel, new GridBagConstraints(1, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(buttonLoad, new GridBagConstraints(2, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 0), 0, 0));
	pane.add(buttonSave, new GridBagConstraints(3, sy, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 0), 0, 0));
	sy++;

	pack();
}

/** Kopiert die Komponenten nach IO. */
private void comp2var(){
	IO.creator = textCreator.getText();
	author = textAuthor.getText();
	IO.resize = checkboxResize.getState();
	IO.docking = checkboxDocking.getState();
	IO.slomo = checkboxSLOMO.getState();
	IO.hamsterimg = checkboxHamster.getState();
	IO.systemdialogs = checkboxSystemDialogs.getState();
	IO.standardloader = checkboxStandardLoader.getState();
	IO.sleep = sliderSleep.getValue()*10;
	IO.size = sliderSize.getValue();
}

/** Kopiert IO in die Komponenten. */
private void var2comp(){
	textCreator.setText(IO.creator);
	textAuthor.setText(author);
	checkboxResize.setState(IO.resize);
	checkboxDocking.setState(IO.docking);
	checkboxSLOMO.setState(IO.slomo);
	checkboxHamster.setState(IO.hamsterimg);
	checkboxSystemDialogs.setState(IO.systemdialogs);
	checkboxStandardLoader.setState(IO.standardloader);
	sliderSleep.setValue(IO.sleep/10);
	labelSleep.setText("Sleep: "+IO.sleep);
	sliderSize.setValue(IO.size);
	labelSize.setText("Size: "+IO.size);
}

/** Kopiert die Komponenten in die Variablen von ControlFrame. */
private void comp2VAR(){

	String a = textCreator.getText();
	if (a.indexOf("\n")>=0){
		// am ersten Zeilenwechsel stoppen
		a = a.substring(0, a.indexOf("\n"));
	}
	ControlFrame.creatorName = a;

	a = textAuthor.getText();
	if (a.indexOf("\n")>=0){
		// am ersten Zeilenwechsel stoppen
		a = a.substring(0, a.indexOf("\n"));
	}
	if (cf!=null && cf.maze!=null)
		cf.maze.author = a;

	ControlFrame.engineAutoResize = checkboxResize.getState();
	ControlFrame.engineDocking = checkboxDocking.getState();
	EngineX.SLO_MO = checkboxSLOMO.getState();
	EngineX.showHamsterImg = checkboxHamster.getState();
	ControlFrame.systemDialogs = checkboxSystemDialogs.getState();
	ControlFrame.standardLoader = checkboxStandardLoader.getState();
	ControlFrame.defaultSleep = sliderSleep.getValue()*10;
	ControlFrame.defaultCellSize = sliderSize.getValue();
}

/**
 *
 * @param ev java.awt.event.ActionEvent
 */
public void actionPerformed(ActionEvent ev) {
try{
	if (ev.getSource()==buttonCancel){
		dispose();
		cancelPressed();
		return;
	}
	if (ev.getSource()==buttonOK){
		// Die Variablen des ControlFrame setzen
		// und dann okPressed
		comp2VAR();
		dispose();
		okPressed();
		return;
	}
	if (ev.getSource()==buttonLoad){
		// Datei laden und IO in die Komponenten kopieren
		IO.loadState();
		var2comp();
		return;
	}
	if (ev.getSource()==buttonSave){
		// die Komponenten nach IO kopieren und Datei speichern
		comp2var();
		IO.saveState();
		return;
	}
}catch(Exception exc){
	log(exc.toString());
	exc.printStackTrace(System.out);
}
}

/**
 * Ausgabe von Nachrichten des Fensters.
 * Wenn ein ControlFrame vorhanden ist, wird dessen log aufgerufen.
 */
public void log(String s){
	if (cf!=null)
		cf.log(s);
	else
		System.out.print(s);
}

/**
 * Ruft {@link #log(java.lang.String)} auf und haengt einen Zeilenwechsel an.
 */
public void logln(String s){
	log(s+"\n");
}

/**
 * Wird aufgerufen, wenn Cancel gedrueckt wurde.
 */
public void cancelPressed() {
	if (cf!=null) cf.properties = null;
}

/**
 * Wird aufgerufen, wenn OK gedrueckt wurde.
 */
public void okPressed() {
	if (cf!=null){
		cf.properties = null;
		if (cf.engineX!=null)
			cf.engineX.drawMaze();
	}
}

/**
 * Reagiert auf Bewegungen der Slider.
 */
public void stateChanged(ChangeEvent ev) {
	if (ev.getSource()==sliderSleep){
		labelSleep.setText("Sleep: "+sliderSleep.getValue()*10);
		return;
	}
	if (ev.getSource()==sliderSize){
		labelSize.setText("Size: "+sliderSize.getValue());
		return;
	}
}

/**
 * Testroutine.
 * @param args java.lang.String[]
 */
public static void main(String[] args) {
	new PropertiesDialog("Properties", null){
		public void okPressed() {
			super.okPressed();
			System.exit(0);
		}
		public void cancelPressed() {
			super.cancelPressed();
			System.exit(0);
		}
	}
	.show();
}

}