package algds.contest;

import java.io.*;
import java.util.*;

/**
 * Hilfsklasse, die die Properties liest und schreibt.
 *
 * <br>Creation date: (12.02.01 22:50:26)
 * @author Christian Semrau, Stephan Finn
 */
public class PropertiesIO{

	Logable log;
	
	String creator;
	boolean resize, docking, slomo,
	hamsterimg, systemdialogs, standardloader;
	int sleep, size;

/**
 * Setzt Vorgabewerte.
 */
public PropertiesIO(){
	creator = ControlFrame.creatorName;
	resize = ControlFrame.engineAutoResize;
	docking = ControlFrame.engineDocking;
	slomo = EngineX.SLO_MO;
	hamsterimg = EngineX.showHamsterImg;
	systemdialogs = ControlFrame.systemDialogs;
	standardloader = ControlFrame.standardLoader;
	sleep = ControlFrame.defaultSleep;
	size = ControlFrame.defaultCellSize;
}


/**
 * Setzt die Variablen auf die uebergebenen Werten.
 */
public PropertiesIO(String creator,
	boolean resize, boolean docking, boolean slomo,
	boolean hamsterimg, boolean systemdialogs, boolean standardloader,
	int sleep, int size) {

	this.creator = creator;
	this.resize = resize;
	this.docking = docking;
	this.slomo = slomo;
	this.hamsterimg = hamsterimg;
	this.systemdialogs = systemdialogs;
	this.standardloader = standardloader;
	this.sleep = sleep;
	this.size = size;
}

/**
 * Ausgabe von Nachrichten des Fensters.
 * Kann ueberschrieben werden, um sie umzuleiten.
 */
public void log(String s){
	if (log==null)
		System.out.print(s);
	else
		log.log(s);
}

/**
 * Ruft {@link #log(java.lang.String)} auf und haengt einen Zeilenwechsel an.
 */
public void logln(String s){
	log(s+"\n");
}

/**
 * Laedt die Einstellungen aus einer Ini-Datei.
 * Speichert die Werte in den Komponenten des Objekts.
 */
public boolean loadState() {
	// komplette Datei einlesen
	// Leerzeilen werden uebersprungen!
	String[] values = Utils.readStringArray("world.ini",true);

	// bei Fehlern wird null als Feld geliefert
	if (values == null){
		logln("Cannot read world.ini");
		return false;
	}
	if (values.length<10){
		logln("Unexpected end of ini file. ("+values.length+" lines)");
		return false;
	}

	// Wenn ein boolean-Wert nicht "true" ist, wird er als false
	// angenommen
	int i = 1; // erste Zeile ueberspringen

	// erstes Zeichen '.' ueberspringen
	creator = (values[i++].substring(1));
	// author wird nicht gespeichert
	slomo = values[i++].equalsIgnoreCase("true");
	resize = values[i++].equalsIgnoreCase("true");
	docking = values[i++].equalsIgnoreCase("true");
	hamsterimg = values[i++].equalsIgnoreCase("true");
	systemdialogs = values[i++].equalsIgnoreCase("true");
	standardloader = values[i++].equalsIgnoreCase("true");

	try{
		sleep = Integer.parseInt(values[i++]);
	}catch(NumberFormatException exc){
		// sleep nicht aendern
	}

	try{
		size = Integer.parseInt(values[i++]);
	}catch(NumberFormatException exc){
		// size nicht aendern
	}

	//logln("Properties loaded.");
	return true;
}

/**
 * Speichert die Einstellungen in einer Datei.
 * Die Werte der Komponenten des Objekts werden geschrieben.
 */
public boolean saveState() {
	String values[]= {
		// einen Punkt davor setzen, weil Leerzeilen beim Einlesen
		// uebersprungen werden
		"#Felder:Creator,SloMo,Resize,Docking,HamsterImg,SystemDialogs,StandardLoader,Sleep,CellSize",
		"."+creator,
		(slomo?"true":"false"),
		(resize?"true":"false"),
		(docking?"true":"false"),
		(hamsterimg?"true":"false"),
		(systemdialogs?"true":"false"),
		(standardloader?"true":"false"),
		(""+sleep),
		(""+size)
	};

	try {

	BufferedWriter bw = new BufferedWriter(new FileWriter("world.ini"));
	for (int i = 0; i < values.length; i++) {
		bw.write(values[i]);
		bw.newLine();
	}
	bw.close();

	} catch (IOException ex) {
		logln("Cannot save properties! "+ex.getMessage());
		return false;
	}
	logln("Properties saved.");
	return true;
}

}