package algds.contest;

import java.io.*;

/**
 * Der Thread, in dem der Hamster laeuft.
 * <br>Creation date: (03.02.01 01:07:51)
 * @author Christian Semrau, Stephan Finn
 */
public final class Runner extends Thread {
	MazeEngineImpl engine;

/**
 * Runner constructor comment.
 */
public Runner(MazeEngineImpl eng) {
	super();
	engine = eng;
}

/**
 * Startet den Hamster und wartet, bis er fertig ist.
 * Dann wird der waiter aufgeweckt.
 * @see java.lang.Runnable
 */
public void run() {
	/*
	* engine.exceptionString muss einen "\n" am Anfang und Ende haben.
	*/
	engine.exceptionString = null;
	engine.hamsterException = null;

	// Startet den Hamster, gut eingepackt in einen try/catch-Block
	int exception = 0;
	try {

		engine.hamster.run();

	} catch (HamsterStopException exc) {
	// Hamster wurde von aussen gestoppt
		// keine Strafpunkte
		engine.exceptionString = "\n"+exc+"\n";
		engine.log(engine.exceptionString);
		exception = 1;
	} catch (HamsterDiedException exc) {
	// Hamster ist verhungert
		engine.extraPoints += engine.POINTS_DIE;
		engine.exceptionString = "\n"+exc+"\n"
			+"Control: Strafe fuer Verhungern!\n";
		engine.log(engine.exceptionString);
		exception = 2;
	} catch (Exception exc) {
	// andere Exception
		engine.extraPoints += engine.POINTS_CRASH;
		// die hamsterException setzen, die von MazeRunner
		// bei der Ausgabe mitgespeichert wird
		engine.hamsterException = exc;
		engine.exceptionString = "\n"+exc+"\n"
			+"Control: Strafe fuer Exception!\n";
		engine.log(engine.exceptionString);
		// Exception wird mit der log-Methode der engine ausgegeben
		// anders kann ich den StackTrace nicht umleiten
		exc.printStackTrace(new PrintWriter(new LogWriter(engine)));
		exception = 3;
	}

	if (exception == 0) {
		// keine Exception
		if (!engine.atHome()){
			// nicht zuhause beim Beenden
			engine.extraPoints += engine.POINTS_NOTHOME;
			engine.exceptionString = "\n"
				+"Control: Strafe fuer Beenden "
				+"ausserhalb des Heimatfeldes!\n";
			engine.log(engine.exceptionString);
		}
	}

	engine.running = false;
	engine.stopWatch();

	/*
	 * Hier tritt ein Problem bei sehr kurzen und rechenintensiven 
	 * Hamstern auf, die durch z.B. den MazeRunner gestartet wurden.
	 * ---- Der Hamster wird vom MazeRunner aus gestartet und danach
	 * wird die Semaphore "lock" auf Warten gesetzt um auf die Beendigung
	 * des Hamsters zu warten. Dieser ist jedoch so schnell fertig, dass
	 * der MazeRunner es in der Zwischenzeit nicht schafft sein "lock.wait"
	 * auszufuehren.
	 * Daraus folgt, dass der "Runner" schon sein "notify" gegeben hat, 
	 * wenn der "MazeRunner" noch nicht mal sein "wait" ausgefuehrt hat.
	 * Daraus folgt wieder, dass dieser wohl bis in alle Ewigkeit auf
	 * sein "notify" warten muss.
	 * Desshalb an dieser Stelle ein "Thread.sleep()", damit der 
	 * "MazeRunner" erst mal sein "wait" machen kann. 
	 */ 
	try{
	    Thread.sleep(1000);
	}catch(InterruptedException exc){
	}

	/*
	 * Das "waiter.notify" wird ab jetzt in "engine.hamsterStopped()" 
	 * aufgerufen um der "watch" auch die Moeglichkeit zu geben, 
	 * das Programm zu beenden
	 * ---
	 * die watch ruft ueber "ControlFrame.stopHamster(true, desc)"
	 * die Methode "Runner.timeOut()" auf, die urspruenglich auch
	 * das "notify" enthielt
	 */

	engine.hamsterStopped();
}

/**
 * Stoppt den Hamster. Wird von der watch und von
 * {@link ControlFrame#buttonStop_actionPerformed()} aufgerufen.
 * Ruft dazu die Methode Thread.stop() auf (deprecated - aber nicht anders
 * zu machen).
 */
public void timeOut(String desc){
	engine.extraPoints += engine.POINTS_CRASH;
	engine.exceptionString = "\n"
		+"Control: Strafe fuer "+desc+" Timeout!\n";
	engine.log(engine.exceptionString);

	engine.running = false;
	engine.stopWatch();

	/*
	 * Hier tritt das Problem mit den zu schnellen Hamstern wohl nicht auf.
	 * Ein Thread.sleep koennen wir uns also hier ersparen?
	 */

	engine.hamsterStopped();
	
	// Thread stoppen
	stop();
}

}