package algds.contest;

import java.awt.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;

/**
 * Hilfsmethoden.
 * @author Christian Semrau, Stephan Finn
 */
public final class Utils {

	/** systemspezifisches Zeichen fuer den Zeilenwechsel. */
	static String lineSeparator = System.getProperty("line.separator");

	/** Ziffern fuer HEX-Darstellung.
	 * @see #hexString(int)
	 */
	static char[] ziffern = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

/*
	public static int stringToInt(String s){
	        // hier gibt es schon wen, der das kann.
		return Integer.parseInt(s);
	}
*/

/**
 * Wandelt eine Zahl in einen Hex-String um.
 * Behandlung negativer Zahlen eingebaut.
 */
public static String hexString(int number){
	if (number==0) return "0";
	String sgn = number<0 ? "-" : "";
	if (number<0) number = -number;
	String s = "";

	while(number>0){
		s = ziffern[number%16] + s;
		number = number/16;
	}
	// Das gleiche Ergebnis wie die while-Schleife erzielt
	// diese Anweisung
	// s = Integer.toHexString(number).toUpperCase();

	return sgn+s;
}

/** Erzeugt ein GridBagConstraints-Object.
 * Dasselbe Ergebnis kann auch direkt durch einen Konstruktoraufruf
 * erreicht werden.
 */
public static GridBagConstraints gbc(int gridx, int gridy,
	int gridwidth, int gridheight,
	double weightx, double weighty,
	int anchor, int fill,
	Insets insets, int ipadx, int ipady) {

	GridBagConstraints t = new GridBagConstraints();
	t.gridx = gridx;
	t.gridy = gridy;
	t.gridwidth = gridwidth;
	t.gridheight = gridheight;
	t.fill = fill;
	t.ipadx = ipadx;
	t.ipady = ipady;
	t.insets = insets;
	t.anchor  = anchor;
	t.weightx = weightx;
	t.weighty = weighty;
	return t;
}

/**
 * Liest eine angegebene Textdatei ein
 * und gibt diese in einem String-Feld zur&uuml;ck.
 * Im Fehlerfall wird <code>null</code> geliefert.
 *
 * @return String[] jede Zeile in einem String
 * @param filename Der Name (inkl. Pfad) der einzulesenden Datei
 * @param skipEmpty Leerzeilen ueberspringen
 */
public static String[] readStringArray (String filename, boolean skipEmpty) {
  String[] result = null;
  Vector values = new Vector ();
  try {
	BufferedReader reader =
	  new BufferedReader (new FileReader (filename));
	while (reader.ready ()) {
	  String value = reader.readLine ();
	  if (value!=null && (!skipEmpty || value.length()>0))
		  values.addElement (value);
	}
	reader.close ();
	result = new String[values.size ()];
	for (int i = 0; i < values.size (); i++)
	  result[i] = (String) values.elementAt (i);
  }
  catch (IOException e) {
	System.out.println ("I/O Exception: " + e.getMessage ());
  }
  return result;
}

/**
 * Liest in der angegebenen Zipdatei den ersten Eintrag ein
 * und gibt diesen in einem String-Feld zur&uuml;ck.
 * Im Fehlerfall wird <code>null</code> geliefert.
 * Leerzeilen werden uebersprungen!
 *
 * @return String[] jede Zeile in einem String
 * @param filename Der Name (inkl. Pfad) der einzulesenden Datei
 */
public static String[] readZipStringArray (String filename, String entryname) {
  String[] result = null;
  Vector values = new Vector ();
  try {
	FileInputStream in = new FileInputStream(filename);
	ZipInputStream zi = new ZipInputStream(in);
	InputStreamReader ri = new InputStreamReader(zi);
	BufferedReader reader = new BufferedReader(ri);
	ZipEntry ze=zi.getNextEntry();

	if (entryname!="")
		while (ze!=null && !ze.getName().equals(entryname))
			ze=zi.getNextEntry();

	if (ze!=null)
	while (reader.ready ()) {
		String value = reader.readLine ();
		if (value!=null && value.length()>0)
			values.addElement (value);
	}

	reader.close ();
	result = new String[values.size ()];
	for (int i = 0; i < values.size (); i++)
	  result[i] = (String) values.elementAt (i);
  }
  catch (ZipException e) {
	System.out.println ("Zip Exception: " + e.getMessage ());
  }
  catch (IOException e) {
	System.out.println ("I/O Exception: " + e.getMessage ());
  }
  return result;
}

public static String[] readZipEntries (String filename) {
  String[] result = null;
  Vector values = new Vector ();
  try {
	FileInputStream in = new FileInputStream(filename);
	ZipInputStream zi = new ZipInputStream(in);
	InputStreamReader ri = new InputStreamReader(zi);
	BufferedReader reader = new BufferedReader(ri);
	ZipEntry ze;

	while ((ze=zi.getNextEntry())!=null)
	if (!ze.isDirectory()){
		values.addElement (ze.getName());
	}

	reader.close ();
	result = new String[values.size ()];
	for (int i = 0; i < values.size (); i++)
	  result[i] = (String) values.elementAt (i);
  }
  catch (ZipException e) {
	System.out.println ("Zip Exception: " + e.getMessage ());
  }
  catch (IOException e) {
	System.out.println ("I/O Exception: " + e.getMessage ());
  }
  return result;
}


/**
 * Laedt einen Hamster neu.
 * Verwendet dazu den {@link FileClassLoader}.
 */
public static Hamster loadHamsterForName(String name, Logable log){
	try{
		// Dadurch, dass jedesmal ein neuer ClassLoader angelegt
		// wird, werden die Klassen stets neu geladen.
		Class c = new FileClassLoader("hamster").loadClass(name);
		return (Hamster)c.newInstance();
	}catch(Exception exc){
		if (log!=null) log.log(exc+"\n");
		return null;
	}
}

/**
 * Laedt einen Hamster neu.
 * Je nach Wert von {@link ControlFrame#standardLoader} wird der
 * Standard-ClassLoader verwendet oder {@link
 * #loadHamsterForName(java.lang.String, algds.contest.Logable)} aufgerufen.
 */
public static Hamster loadHamster(String name, Logable log){
	try {
		if (!ControlFrame.standardLoader)
		return loadHamsterForName(name,log);
		else
		return (Hamster) Class.forName(name).newInstance();
	} catch (Exception exc) {
		if(log!=null) log.log(exc+"\n");
		return null;
	}
}

public static void writeVector(String name, Vector zeilen)
throws IOException{
	try {
		BufferedWriter bw = new BufferedWriter(new FileWriter(name));

		for (int i = 0; i < zeilen.size(); i++)
			bw.write((String) (zeilen.elementAt(i)));
		bw.close();
	} catch (IOException ex) {
		throw ex;
	}
}

public static void writeVectorZip(String zipname, String entryname, Vector zeilen)
throws IOException{
try {
if (new File(zipname).exists()){

	FileInputStream in = new FileInputStream(zipname);
	ZipInputStream zi = new ZipInputStream(in);

	//System.out.println("ZipFile: "+zipname);
	File tmpdir = new File(zipname).getAbsoluteFile().getParentFile();
	//System.out.println("TmpDir: "+tmpdir);
	File tmpfile = File.createTempFile("maze",".tmp", tmpdir);
	FileOutputStream out = new FileOutputStream(tmpfile);
	ZipOutputStream zo = new ZipOutputStream(out);

	ZipEntry zei, zeo;
	byte[] b = new byte[32000];
	boolean inserted = false;

	while ((zei=zi.getNextEntry())!=null)
	if (zei.getName().equals(entryname)){
		zeo = new ZipEntry(zei);
		zo.putNextEntry(zeo);
		//System.out.println("Overwrite "+zeo);
		for (int i = 0; i < zeilen.size(); i++){
			byte[] b2 = ((String)zeilen.elementAt(i)).getBytes();
			zo.write(b2, 0, b2.length);
			//System.out.print(".");
		}
		//System.out.println();
		inserted = true;
	}else{
		zeo = new ZipEntry(zei);
		zo.putNextEntry(zeo);
		//System.out.println("Copy "+zeo);
		int len = 0;
		while(len>=0){
			len = zi.read(b, 0, b.length);
			if (len>=0)
				zo.write(b, 0, len);
			//System.out.print(".");
		}
		//System.out.println();
	}

	if (!inserted){
		zeo = new ZipEntry(entryname);
		zo.putNextEntry(zeo);
		//System.out.println("Add "+zeo);
		for (int i = 0; i < zeilen.size(); i++){
			byte[] b2 = ((String)zeilen.elementAt(i)).getBytes();
			zo.write(b2, 0, b2.length);
			//System.out.print(".");
		}
		//System.out.println();
	}
	
	zo.close();
	zi.close();

	new File(zipname).delete();
	tmpfile.renameTo(new File(zipname));
}else{

	FileOutputStream out = new FileOutputStream(zipname);
	ZipOutputStream z = new ZipOutputStream(out);
	OutputStreamWriter w = new OutputStreamWriter(z);

	z.putNextEntry(new ZipEntry(entryname));

	for (int i = 0; i < zeilen.size(); i++)
		w.write((String) (zeilen.elementAt(i)));

	w.flush();
	z.closeEntry();
	w.close();
}
} catch (IOException ex) {
	throw ex;
}
}

}
