package algds.contest;
import java.util.Date;

/**
 * Ein Timer, der den Hamster nach der angegebenen Zeit stoppt.
 * @author Stephan Finn, Christian Semrau
 */
public final class Watch implements Runnable
{
	private final long pause;
	private final Thread thread;
	private final MazeEngineImpl mei;
	private long waitInterval;




/** Schaut jeweils nach 1/6 der Timeout-Zeit, ob das Zeitlimit schon
 * ueberschritten wurde.
 */
public void run() {
	boolean stopped = false;
	long currTime, diff;
	do{
		// abwarten
		try {
			Thread.sleep(waitInterval);
		} catch (InterruptedException exp) {
			// stop wurde aufgerufen
			stopped = true;
		}
		// Zeit holen
		currTime = (new Date()).getTime();
		// Zeit seit Start
		diff = currTime - startTime;

		// wenn gestoppt, dann keine Ausgaben mehr machen
		if (stopped) break;

		// wenn mehr als die halbe Zeit vergangen ist
		// und Gesamtzeit nocht nicht ueberschritten wurde
		if (diff >= pause/2 && diff < pause)
			mei.logln("Control: "+desc+" TimeOut in "
			+(pause-diff)/1000+" sek");
		// wenn nur noch ein Intervall kommt,
		// dann Intervallzeit verkuerzen, fuer einen genaueren Abbruch
		if ((pause-diff<=waitInterval) && !shortened){
			waitInterval = waitInterval/5;
			shortened=true;
		}
	// solange die Zeit nicht ueberschritten ist,
	// der Hamster seine Engine noch hat,
	// und dieser Thread nicht mit stop() unterbrochen wurde
	}while (diff < pause && mei.hamster.isEngineAlive() && !stopped);

	if (diff >= pause && !stopped){
	// wenn die Zeit ueberschritten wurde und der Thread nicht vom
	// User gestoppt wurde
//		mei.log("\nControl: TimeOut!!!\n\n");
		mei.stopHamster(true, desc);
	}
}

/** Setzt {@link Hamster#watchTime} und startet den Thread. */
public void start() {
	thread.start();
}

/** Stoppt den Thread. */
public void stop() {
	thread.interrupt();
}

	private final String desc;
	private boolean shortened;
	/**
	 * Enthaelt den Zeitpunkt des letzten Aufrufs einer
	 * Bewegungsmethode oder den Zeitpunkt des Starts.
	 * Dient zum Abfangen von Endlosschleifen und zur Kontrolle
	 * der Gesamtlaufzeit, indem nach Ablauf einer bestimmten
	 * Zeitspanne der Inaktivitaet der Hamster gestoppt wird.
	 *
	 * @see Constants#INFINITE_LOOP
	 * @see Constants#TOTAL_RUNTIME
	 */
	long startTime = 0;

public Watch(MazeEngineImpl MEI, long PAUSE, String DESC) {
	mei = MEI;
	pause = PAUSE;
	waitInterval = PAUSE/6+100;
	shortened = false;
	desc = DESC==null?"":DESC;

	thread = new Thread(this);
	thread.setName("Watch - wait "+(PAUSE/1000)+" sec");
	thread.setPriority(7);
}
}